/* vsiptest_blockfind_f.c Version 0.000 May 6, 1997
// 
// This file contains the vsiptest_blockfind_f test functionality.
//
// Functional Description:
//   vsip_blockfind_f returns a pointer to the public data array
//   if vsip_block_f *b is public, otherwise it returns a null pointer
//
// Copyright May 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Thomas Egolf
//         egolf@ece.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//      various sizes using the malloc function call and then bind the
//      public block to a private block using the function vsip_blockbind_f.
//      It will then call vsip_blockfind_f to obtain the pointer to the block.
//      After obtaining the pointer, it will compare values in this array to
//      verify that we have obtained the correct pointer. Also a set of
//      tests will create private blocks using the vsip_blockcreate_f and
//      vsip_cblockcreate_f functions and test whether the value returned
//      by vsip_blockfind_f is null.
//
//      Check some of the parameters of this function
//        1) Check the public/private bit (should be set to public (1) if a
//           non-null pointer is returned)
//        2) Check the pointer to the array that was returned from the function
//           and verify that the data is the same as initially used to fill
//           the array.
//
// VSIP functions called: vsip_blockbind_f, vsip_blockfind_f,
//                        vsip_blockcreate_f, vsip_blockdestroy_f,
//                        vsip_cblockcreate_f
//                        
//
// Revision History:
// May 6, 1997  Added copyright notice.             
// May 6, 1997  Executed tests 1-2 without error and block sizes up to 5000 */

#include<stdio.h>
#include<stdlib.h>  /* For malloc functionality */
#include "vsiptest_block.h"

int main()
{

  size_t                block_size, i;
  vsip_scalar_f        *public_block;
  vsip_memory_hint      hint = 0;
  vsip_scalar_f        *find_block;
  vsip_block_f         *private_block, *bind_block;
  int                   j, ret = 0;

  vsip_init ((void *)0); 
  /* Check various public block sizes from 2 to block_length_max */
  for (j=0; j<elements(VSIPTEST_BLOCKFIND_F_BLOCK_SIZE); j++)
  {
    block_size = VSIPTEST_BLOCKFIND_F_BLOCK_SIZE[j];
    public_block = (vsip_scalar_f*)malloc(block_size*sizeof(vsip_scalar_f));

    /* Fill the block with data values equal to its index value */
    for(i=0; i<block_size; i++) 
    {
      *(public_block + i) = (vsip_scalar_f)i;
    }

    /********************* VSIP_BLOCKBIND_D  ************************
      Bind to the public block with at least block_size real elements. */
    bind_block = vsip_blockbind_f(public_block, block_size, hint);

    /********************* VSIP_BLOCKFIND_D *************************
      Find the public block pointed to by bind_block */
    find_block = vsip_blockfind_f(bind_block);

    if(find_block == (vsip_scalar_f*)NULL) 
    {
      /* Report error */
      vtprintf("Error in vsip_blockfind_f(); (1) \n"
               "Pointer to public block was not returned from find_block \n");
      ret |= 1<<1;
      break;
    }
    else 
    {
      /* Check the data array's values for matching with public_block */
      for(i=0; i<block_size; i++) 
      {
        if(vtfcomp (*(public_block + i), *(find_block + i)) )
        {
          /* Report error */
          vtprintf("Error in vsip_blockfind_f(); (2) \n"
                   "Array values from found block do not match original \n");
          ret |= 1<<2;
          break;
        }
      }
    }

    free(public_block);

    /********************* VSIP_BLOCKCREATE_D ***********************
      Create a private real-valued data block */
    private_block = vsip_blockcreate_f(block_size, hint);

    /********************* VSIP_BLOCKFIND_D *************************
      Find if a call to block_find will return a public block in
      this case */
    find_block = vsip_blockfind_f(private_block);

    if(find_block != NULL) {
      /* Report error */
      vtprintf("Error in vsip_blockfind_f(); (3) \n"
               "Pointer to VSIP block was returned from find_block \n");
      ret |= 1<<3;
      break;
    }

    /******************** VSIP_BLOCKDESTROY_D ***********************
      Destroy the block after checking all the attributes */
    vsip_blockdestroy_f(private_block);

  }
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
