/* vsiptest_blockadmit_f.c Version 0.000 March 26, 1999
//
// This file contains the vsiptest_blockadmit_f test functionality.
//
// Functional Description:
//   vsip_blockadmit_f "admits" a VSIPL block object, vsip_dblock_p,
//   for VSIPL operations on the associated views.
//
// Copyright March 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//
// Test description:
 *    Create valid VSIPL block and admit it.
 *     Admit it again.
 *     Destroy block and try to admit it.
 *     Update flag will not be tested in this routine, but in release routine.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements after binding
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for binding for negative input lengths (error conditions)
//
// VSIP functions called: vsip_blockadmit_f, vsip_vbind_f, vsip_vget_f,
//                        vsip_vgetlength_f, vsip_vdestroy_f
//
// Revision History:
// March 26, 1999  Created */


#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  size_t            block_size = VSIPTEST_BLOCKADMIT_BLOCK_SIZE;
  vsip_memory_hint  hint = 0;
  vsip_cblock_f    *block;
  vsip_scalar_bl    update = 1;
  int               ret = 0;
  int               i;
  vsip_cvview_f    *cview;
  vsip_vview_f     *iview;
  vsip_vview_f     *rview;

  static vsip_scalar_f  *array1, *array2;
 
  vsip_init ((void *)0);

  array1 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));

  block = vsip_cblockbind_f(array1, array2, block_size, hint);
  if (block == NULL)
  {
    vtprintf ("Error in vsip_cblockadmit_f(); (1)\n"
              "Failed to bind complex block for test.  Test #2 not run\n");
    ret |= 1<<1;
  }

  /* Test update flag */
  for (i=0; i<block_size; i++)
  {
    array1[i] = (vsip_scalar_f)i;
    array2[i] = (vsip_scalar_f)i * 2;
  }

    /********************* VSIP_CBLOCKADMIT_f ****************************/
    /* split */
  if (!block || vsip_cblockadmit_f(block, update))
  {
    vtprintf ("Error in vsip_cblockadmit_f(); (2)\n"
              "Failed to admit split block of size %i. \n", block_size);
    ret |= 1<<2;
  }
  
  iview = NULL;
  rview = NULL;
  cview = NULL;

  cview = vsip_cvbind_f (block, 0, 1, block_size);
  if (cview)
  {
    iview = vsip_vimagview_f (cview);
    rview = vsip_vrealview_f (cview);
  }

  for (i=0; i<block_size; i++)
  {
    if (iview != NULL && rview != NULL)
    {
      if (
          vtfcomp(vsip_vget_f (rview, i), (vsip_scalar_f)i ) ||
          vtfcomp(vsip_vget_f (iview, i), (vsip_scalar_f)i * 2 )
         )
      {
        vtprintf ("Error in vsip_cblockadmit_f(); (3)\n"
                  "Element %i of admitted and updated split "
                  "block did not match array.\n", i);
        ret |= 1<<3;
        break;
      }
    }
  }
  if (cview) vsip_cvdestroy_f (cview);
  if (rview) vsip_vdestroy_f (rview);
  if (iview) vsip_vdestroy_f (iview);
  vsip_cblockdestroy_f (block);

  block = vsip_cblockbind_f (array1, NULL, block_size/2, hint);
  vsip_cblockadmit_f (block, 1);
  if (block == NULL)
  {
    vtprintf ("Error in vsip_cblockadmit_f(); (4)\n"
              "Failed to create interleaved block of "
        "size %i.\n  Test #4 not run.", block_size/2);
    ret |= 1<<4;
  }

  if (!block || vsip_cblockadmit_f(block, update))
  {
    vtprintf ("Error in vsip_cblockadmit_f(); (5)\n"
              "Failed to admit interleaved block of size %i. \n", block_size);
    ret |= 1<<5;
  }

  cview = NULL;
  iview = NULL;
  rview = NULL;

  cview = vsip_cvbind_f (block, 0, 1, block_size/2);
  if (cview)
  {
    iview = vsip_vimagview_f (cview);
    rview = vsip_vrealview_f (cview);
  }
  for (i=0; i<block_size/2; i++)
  {

    if (iview != NULL && rview != NULL)
    {
      if (
          vtfcomp (vsip_vget_f (rview, i), (vsip_scalar_f)i * 2 ) ||
          vtfcomp (vsip_vget_f (iview, i), (vsip_scalar_f)i * 2 + 1 )
         )
      {
        vtprintf ("Error in vsip_cblockadmit_f(); (6)\n"
                  "Element %i of admitted and updated interleaved "
                  "block did not match array.\n", i);
        ret |= 1<<6;
        break;
      }
    }
  }
  if (cview) vsip_cvdestroy_f (cview);
  if (rview) vsip_vdestroy_f (rview);
  if (iview) vsip_vdestroy_f (iview);

  /*-----------------3/26/99 2:50PM-------------------
   * If admit is successful, try to admit it again.
   * --------------------------------------------------*/
  if (vsip_cblockadmit_f(block, update))
  {
    vtprintf ("Error in vsip_cblockadmit_f(); (7)\n"
             "Failed readmit of valid admitted 5000-el block. \n");
    ret |= 1<<7;
  }

  vsip_cblockdestroy_f (block);


  /*-----------------3/29/99 4:28PM-------------------
   * Create known invalid block and try to admit it.
   * --------------------------------------------------*/
  block = vsip_cblockbind_f(NULL, NULL, 1, 0);
  if ( block != NULL)
  {
    if (!vsip_cblockadmit_f(block, update))
    {
      vtprintf ("Error in vsip_cblockadmit_f(); (8)\n"
               "Admitted block with data bound to NULL. \n");
      ret |= 1<<8;
    }
  }
  vsip_cblockdestroy_f (block);
    
#if 0
  block = vsip_cblockcreate_f (1, 0);
  if (!vsip_cblockadmit_f(block, 0))
  {
    vtprintf ("Error in vsip_cblockadmit_f(); (9)\n"
              "Admitted VSIP block. \n");
    ret |= 1<<9;
  }
  vsip_cblockdestroy_f (block);
#endif


  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
