/* vsiptest_cblockcreate_f.c Version 0.000 April 9, 1999
//
// This file contains the vsiptest_cblockcreate_f test functionality.
//
// Functional Description:
//   vsip_cblockcreate_f creates a private data array with N
//   vsip_cscalar_f real scalar elements.
//
//
// Copyright March 1999, Georgia Tech Research Institute.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//         Dan Campbell [adapted to c_*f
//         dan.campbell@gtri.gatech.edu
//         
//
// Test description: This test suite will create real blocks of data with
//      various sizes and with various hints as input. Currently the hints
//      are not supported.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for block sizes of negative numbers (error conditions)
//
// VSIP functions called: vsip_blockcreate_f, vsip_vbind_f, vsip_vramp_f,
//                        vsip_vminval_f, vsip_vmaxval_f, vsip_vgetlength_f,
//                        vsip_vdestroy_f, vsip_blockdestroy_f
//
// Revision History:
// May    5, 1997  Added copyright notice.
// May    6, 1997  Executed tests 1-4 without error and block sizes up to 5000 
// April  9, 1999  Adapted to c*_f
*/

#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  size_t           block_size;
  vsip_memory_hint hint = 0;
  vsip_cblock_f   *block;
  vsip_cvview_f   *blockview;
  vsip_vview_f    *rview, *iview;
  vsip_scalar_f    minval, maxval;
  vsip_cvattr_f    cattrib;
  vsip_length      length;
  int              i, ret = 0;

  vsip_init ((void *)0);

  for (i=0; i<elements(VSIPTEST_CBLOCKCREATE_F_BLOCK_SIZE); i++)
  {
    block_size=VSIPTEST_CBLOCKCREATE_F_BLOCK_SIZE[i];

    /********************* VSIP_BLOCKCREATE_f ***********************
      Create the block */
    block = vsip_cblockcreate_f(block_size, hint);

    /********************* VSIP_VBIND_f *****************************
      Create a view on the block */
    blockview = vsip_cvbind_f(block, 0, 1, (vsip_length)block_size);
    rview = vsip_vrealview_f (blockview);
    iview = vsip_vimagview_f (blockview);

    /********************* VSIP_VRAMP_f *****************************
      Fill the vector view with a ramp from 0 to block_size-1 */
    vsip_vramp_f(0.0,1.0,iview);
    vsip_vramp_f(0.0,1.0,rview);

    /* Check some of the parameters of this function
    //    1) Check the first and last array elements
    //    2) Check the size of the block */

    /********************* VSIP_VMINVAL_f *****************************
     ********************* VSIP_VMAXVAL_f *****************************
      Check the minimum and maximum values of the view on the block */
    minval = vsip_vget_f(rview, 0);
    maxval = vsip_vget_f(rview, block_size-1);

    if(vtfcomp (minval, (vsip_scalar_f)0.0) )
    {
      vtprintf("Error in vsip_cblockcreate_f(); (1) \n"
               "First value assignment does not match first element value"
               " in real view\n");
      ret |= 1<<1;
      break;
    }

    if(vtfcomp (maxval, (vsip_scalar_f)(block_size-1)) )
    {
      vtprintf("Error in vsip_cblockcreate_f(); (2) \n"
               "Last value assignment does not match last element value"
               " in real view\n");
      ret |= 1<<2;
      break;
    }

    minval = vsip_vget_f(iview, 0);
    maxval = vsip_vget_f(iview, block_size-1);

    if(vtfcomp (minval, (vsip_scalar_f)0.0) )
    {
      vtprintf("Error in vsip_cblockcreate_f(); (3) \n"
               "First value assignment does not match first element value"
               " in imaginary view\n");
      ret |= 1<<3;
      break;
    }

    if(vtfcomp (maxval, (vsip_scalar_f)(block_size-1)) )
    {
      vtprintf("Error in vsip_cblockcreate_f(); (4) \n"
               "Last value assignment does not match last element value"
               " in imaginary view\n");
      ret |= 1<<4;
      break;
    }

    /********************* VSIP_VGETLENGTH_f ************************
      Check the length of the block allocated */

  /*-----------------3/26/99 10:36AM------------------
   * Use getattribs for checking length
   * --------------------------------------------------*/

    vsip_cvgetattrib_f (blockview, &cattrib);
    length = cattrib.length;
    if(length != (vsip_length)block_size)
    {
      /* Report error */
      vtprintf("Error in vsip_cblockcreate_f(); (5) \n"
               "Size does not match allocated size \n");
      ret |= 1<<5;
      break;
    }

    /********************* VSIP_VDESTROY_f ****************************
     ********************* VSIP_BLOCKDESTROY_f ************************
      Destroy the block after checking all the attributes */

  /*-----------------3/26/99 10:37AM------------------
   *  Do a blockdestroy, then bind a view to destroyed block and see
   *  if it fails.
   * --------------------------------------------------*/

    vsip_cblockdestroy_f(vsip_cvdestroy_f(blockview));

  }
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
