/* vsiptest_complexbinary.c Version 0.000 September 2, 1999
// 
// This file contains the source code included for testing complex binary
// element wise functions
// 
// Copyright September 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) with various stride combinations, test each of the 3 
//         possible types of complex blocks
//         V    a) VSIP block
//         US   b) User created split complex block
//         UI   c) User created interleaved complex block
//         where ~ is the operator to be tested
//         VR ~ UR = UR2
//      2) Test in-place operation: VR ~ UR = UR
//
// Revision History:
// September 2, 1999   Created
*/

#include <math.h>

/* complex binary */

/* Input #defines:
        INPUT_FILENAME          results vector file
        FUNCTION_NAME           string desc. of function to be tested
        TEST_FUNCTION           pointer to vsip_ function to be tested
        VSIPTEST_EPSILON_MAX    max difference for this test
        VSIPTEST_THOROUGH       defined if thorough tests are to be run
*/


char filename[] = INPUT_FILENAME;

int vttest_function (char *filename, 
                     void (*function) (const vsip_cvview_f *, 
                                       const vsip_cvview_f *, 
                                       const vsip_cvview_f *),
                     vsip_cvview_f *view1, vsip_cvview_f *view2, 
                     vsip_cvview_f *view3, int length, 
                     char *function_name, int first_error)
{
  FILE *fp = fopen (filename, "r");
  int ret = 0;
  int read = 0;
  int val = 0;
  int i;
  vsip_scalar_f  rrval, irval, val1, val2, val3, val4, val5, val6;
  vsip_cscalar_f rval, cval1, cval2;

  if (!fp)
  {
    vtprintf ("Error in %s (%i)\n"
              "Could not open %s for reading\n", function_name,
              first_error+0, filename);
    ret |= 1<<(first_error+0);
    return (ret); 
  }

  val = vtgetlength (fp);

  if (val < length)
  {
    vtprintf ("Error in %s (%i)\n"
              "length (%i) in data file(%s)\n"
              "is shorter than views for testing(%i)\n", function_name,
              first_error+0, val, filename, length);
    ret |= 1<<(first_error+0);
    fclose (fp);
    return (ret); 
  }

  /* load the operand vectors */
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f,%f %f,%f%*[^\n]\n", 
                   &val1, &val2, &val3, &val4);
    if (read<4)
    {
      vtprintf ("Error in %s (%i)\n"
                "Read %i elements in row %i of data file %s\n"
                "expecting 4\n", function_name, first_error+0, read, i, 
                filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    cval1 = vsip_cmplx_f (val1, val2);
    cval2 = vsip_cmplx_f (val3, val4);
    vsip_cvput_f (view1, i, cval1);
    vsip_cvput_f (view2, i, cval2);
  }
  fclose (fp);
  
  /* do the function */

  function (view1, view2, view3);

  /* check the results */

  fp = fopen (filename, "r");
  vtgetlength (fp);
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f,%f %f,%f %f,%f\n", &val1, &val2, &val3, &val4,
                   &val5, &val6);
    if (read<6)
    {
      vtprintf ("Error in %s (%i)\n"
                "Could not read result value for row %i of "
                "data file %s\n", function_name, first_error+0,
                filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    rval = vsip_cvget_f (view3, i);
    rrval = vsip_real_f (rval);
    irval = vsip_imag_f (rval);

    update_error_element (sqrt((rrval-val5)*(rrval-val5)+(irval-val6)*(irval-val6)));

#ifdef VSIPTEST_ELWISE_DO_EPSILON_TEST

    if (fabs (rrval - val5) > VSIPTEST_EPSILON_MAX ||
        fabs (irval - val6) > VSIPTEST_EPSILON_MAX)
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed closeness test - expected %f,%f "
                "got %f,%f\ninput=(%f,%f:%f,%f)\n", function_name, 
                first_error+1, i, val5, val6, rrval, irval, 
                vsip_real_f (vsip_cvget_f (view1, i)),
                vsip_imag_f (vsip_cvget_f (view1, i)),
                vsip_real_f (vsip_cvget_f (view2, i)),
                vsip_imag_f (vsip_cvget_f (view2, i)));
      ret |= 1<<(first_error+1);
    }
#endif

#ifdef VSIPTEST_ELWISE_DO_SIGN_TEST
    if ( (val5 <  0 && rrval >= 0) ||
         (val5 >  0 && rrval <= 0) ||
         (val5 != 0 && rrval == 0) ||
         (val6 <  0 && irval >= 0) ||
         (val6 >  0 && irval <= 0) ||
         (val6 != 0 && irval == 0) )
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed sign test - expected %f,%f got %f,%f\n"
                "input=(%f,%f:%f,%f)\n",
                function_name, first_error+2, i, val5, val6, 
                rrval, irval, 
                vsip_real_f (vsip_cvget_f (view1, i)),
                vsip_imag_f (vsip_cvget_f (view1, i)),
                vsip_real_f (vsip_cvget_f (view2, i)),
                vsip_imag_f (vsip_cvget_f (view2, i)));
      ret |= 1<<(first_error+2);
    }
#endif

    if (ret)
    {
      fclose (fp);
      return (ret); 
    }
  }
  update_error_total();

  fclose (fp);
  return (ret); 
}

int main(int argc, char *argv[])
{
  int length;
  int ret = 0;
  vsip_cvview_f *V, *US, *UI, *VT;
  vsip_cblock_f *csplitb, *cinterb;
  vsip_scalar_f *array1, *array2, *array3;
  int i;
  vsip_stride lstride, rstride;
  vsip_offset offset;
  vsip_length templen;

  vsip_init ((void *)0);

  length = vtelwise_get_length (filename);
  if (!length)
  {
    vtprintf ("Error in %s (1)\n"
              "Could not get length from datafile %s\n",
              FUNCTION_NAME, filename);
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  V = vsip_cvcreate_f (length, 0);

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*length);

  csplitb = vsip_cblockbind_f (array1, array2, length, 0);
  cinterb = vsip_cblockbind_f (array3, NULL, length,0);

  vsip_cblockadmit_f (csplitb, 0);
  vsip_cblockadmit_f (cinterb, 0);

  error_init(argv[1]);
  if (!V )
  {
    vtprintf ("Error in %s (2)\n"
              "Could not allocate all test vectors\n", FUNCTION_NAME);
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  /* Stride tests */
  for (i=0; i<elements(VSIPTEST_ELWISE_LEFT_STRIDES); i++)
  {
    lstride = VSIPTEST_ELWISE_LEFT_STRIDES[i];
    rstride = VSIPTEST_ELWISE_RIGHT_STRIDES[i];
    if (rstride == 1)
    {
      offset = 0;
    }
    else if (rstride == -1)
    {
      offset = length-1;
    }
    else
    {
      vtprintf ("Illegal stride for right stride of stride test in\n"
                "Must be 1 or -1\n");
      report (-1); 
      vsip_finalize ((void *)0);
      return (-1 != 0);
    }
    templen = length / lstride;
    VT = vsip_cvcreate_f (templen, 0);
    US = vsip_cvbind_f (csplitb, 0,      lstride, templen);
    UI = vsip_cvbind_f (cinterb, offset, rstride, templen);

    if (ret |= vttest_function (filename, TEST_FUNCTION, VT, US, UI,
                                templen, FUNCTION_NAME, 3) )
    {
      vtprintf ("Stride test (%i, %i)->%i produced errors\n", 1, lstride,
                rstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    error_report_test ("Stride test (%i, %i)->%i", 1, lstride, rstride);

    vsip_cvdestroy_f (UI);
    vsip_cvdestroy_f (US);
    vsip_cvalldestroy_f (VT);
  }

#ifndef VSIPTEST_THOROUGH
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
#endif

  US = vsip_cvbind_f (csplitb, 0, 1, length);
  UI = vsip_cvbind_f (cinterb, 0, 1, length);

  /* In place test */

  if (ret |= vttest_function (filename, TEST_FUNCTION, V, US, US, 
                              length, FUNCTION_NAME, 6) )
  {
    vtprintf ("In place test produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("In place test\n");
  error_finish();
  report (ret); 
  vsip_finalize ((void *)0);
  vsip_cvalldestroy_f (UI);
  vsip_cvalldestroy_f (US);
  return (ret != 0);
}
