/* vsiptest_complextoreal.c Version 0.000 September 3, 1999,
// 
// This file contains the source code included for testing element wise
// functions which take complex vectors as input, and produce real
// real vectors as output
// 
// Copyright September 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) with unit stride, test each of the 3 
//         possible types of complex blocks
//         V    a) VSIP block
//         US   b) User created split complex block
//         UI   c) User created interleaved complex block
//         VR   d) VSIP created real block
//         V  = VR
//         US = VR
//         UI = VR
//      2) With various stride combinations, try
//         V  = VR
//
// Revision History:
// September 2, 1999   Created
*/

#include <math.h>

/* Input #defines:
        INPUT_FILENAME          results vector file
        FUNCTION_NAME           string desc. of function to be tested
        TEST_FUNCTION           pointer to vsip_ function to be tested
        VSIPTEST_EPSILON_MAX    max difference for this test
        VSIPTEST_THOROUGH       defined if thorough tests are to be run
*/


char filename[] = INPUT_FILENAME;

int vttest_function (char *filename, 
                     void (*function) (const vsip_cvview_f *, 
                                       const vsip_vview_f *),
                     vsip_cvview_f *view1, vsip_vview_f *view2, 
                     int length, char *function_name, int first_error)
{
  FILE *fp = fopen (filename, "r");
  int ret = 0;
  int read = 0;
  int val = 0;
  int i;
  vsip_scalar_f  val1, val2, val3;
  vsip_scalar_f rval;
  vsip_cscalar_f cval1;

  if (!fp)
  {
    vtprintf ("Error in %s (%i)\n"
              "Could not open %s for reading\n", function_name,
              first_error+0, filename);
    ret |= 1<<(first_error+0);
    return (ret); 
  }

  val = vtgetlength (fp);

  if (val < length)
  {
    vtprintf ("Error in %s (%i)\n"
              "length (%i) in data file(%s)\n"
              "is shorter than views for testing(%i)\n", function_name,
              first_error+0, val, filename, length);
    ret |= 1<<(first_error+0);
    fclose (fp);
    return (ret); 
  }

  /* load the operand vectors */
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f,%f %f%*[^\n]\n", 
                   &val1, &val2, &val3);
    if (read<3)
    {
      vtprintf ("Error in %s (%i)\n"
                "Read %i elements in row %i of data file %s\n"
                "expecting 3\n", function_name, first_error+0, read, 
                i, filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    cval1 = vsip_cmplx_f (val1, val2);
    vsip_cvput_f (view1, i, cval1);
  }
  fclose (fp);
  
  /* do the function */

  function (view1, view2);

  /* check the results */

  fp = fopen (filename, "r");
  vtgetlength (fp);
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f,%f %f\n", &val1, &val2, &val3);
    if (read<3)
    {
      vtprintf ("Error in %s (%i)\n"
                "Could not read result value for row %i of "
                "data file %s\n", function_name, first_error+0,
                filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    rval = vsip_vget_f (view2, i);

    update_error_element (fabs(rval-val3));

#ifdef VSIPTEST_ELWISE_DO_EPSILON_TEST

    if (fabs (rval - val3) > VSIPTEST_EPSILON_MAX)
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed closeness test - expected %f "
                "got %f\ninput=(%f,%f)\n", function_name, 
                first_error+1, i, val3, rval, 
                vsip_real_f (vsip_cvget_f (view1, i)),
                vsip_imag_f (vsip_cvget_f (view1, i)));
      ret |= 1<<(first_error+1);
    }
#endif

#ifdef VSIPTEST_ELWISE_DO_SIGN_TEST
    if ( (val3 <  0 && rval >= 0) ||
         (val3 >  0 && rval <= 0) ||
         (val3 != 0 && rval == 0) )
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed sign test - expected %f got %f\n"
                "input=(%f,%f)\n",
                function_name, first_error+2, i, val3, 
                rval,
                vsip_real_f (vsip_cvget_f (view1, i)),
                vsip_imag_f (vsip_cvget_f (view1, i)));
      ret |= 1<<(first_error+2);
    }
#endif

    if (ret)
    {
      fclose (fp);
      return (ret); 
    }
  }
  update_error_total();

  fclose (fp);
  return (ret); 
}

int main(int argc, char *argv[])
{
  int length;
  int ret = 0;
  vsip_cvview_f *V, *US, *UI;
  vsip_vview_f *VR;
  vsip_cblock_f *csplitb, *cinterb;
  vsip_scalar_f *array1, *array2, *array3;
  int i;
  vsip_stride lstride;
  vsip_length templen;

  vsip_init ((void *)0);

  length = vtelwise_get_length (filename);
  if (!length)
  {
    vtprintf ("Error in %s (1)\n"
              "Could not get length from datafile %s\n",
              FUNCTION_NAME, filename);
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  V = vsip_cvcreate_f (length, 0);
  VR = vsip_vcreate_f (length, 0);

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*length);

  csplitb = vsip_cblockbind_f (array1, array2, length, 0);
  cinterb = vsip_cblockbind_f (array3, NULL, length,0);

  US = vsip_cvbind_f (csplitb, 0, 1, length);
  UI = vsip_cvbind_f (cinterb, 0, 1, length);

  vsip_cblockadmit_f (csplitb, 0);
  vsip_cblockadmit_f (cinterb, 0);

  error_init (argv[1]);
  if (!V)
  {
    vtprintf ("Error in %s (2)\n"
              "Could not allocate all test vectors\n", FUNCTION_NAME);
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (ret |= vttest_function (filename, TEST_FUNCTION, V, VR, length, 
                                FUNCTION_NAME, 3) )
  {
    vtprintf ("VSIP created complex block testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("VSIP created complex block test");
 

#ifndef VSIPTEST_THOROUGH
  report (ret); 
    vsip_finalize ((void *)0);
  return (ret != 0);
#endif

  if (ret |= vttest_function (filename, TEST_FUNCTION, US, VR, length, 
                                FUNCTION_NAME, 6) )
  {
    vtprintf ("User created split complex block testing "
              "produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("User created split complex block test");

  if (ret |= vttest_function (filename, TEST_FUNCTION, UI, VR, length, 
                                FUNCTION_NAME, 9) )
  {
    vtprintf ("User created interleaved complex block testing "
              "produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("User created interleaved complex block test");

  vsip_cvalldestroy_f (UI);
  vsip_cvdestroy_f (US);
  vsip_valldestroy_f (VR);

  /* Stride tests */
  for (i=0; i<elements(VSIPTEST_ELWISE_LEFT_STRIDES); i++)
  {
    lstride = VSIPTEST_ELWISE_LEFT_STRIDES[i];
    
    templen = length / lstride;
    US = vsip_cvbind_f (csplitb, 0,      lstride, templen);
    VR = vsip_vcreate_f (templen, 0);

    if (ret |= vttest_function (filename, TEST_FUNCTION, US, VR,
                                templen, FUNCTION_NAME, 12 ))
    {
      vtprintf ("Stride test (%i) produced errors\n", lstride);
      report (ret); 
    vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("Stride test (%i)", lstride);

    vsip_cvdestroy_f (US);
    vsip_valldestroy_f (VR);
  }
  error_finish();
  report (ret); 

  vsip_cblockdestroy_f (csplitb);
  free (array1);
  free (array2);
  free (array3);

  vsip_finalize ((void *)0);
  return (ret != 0);
}
