/* vsiptest_rcvmul_f.c Version 0.000 September 4, 1999,
// 
// This file contains the source code included for testing 
// vsip_rcvmul_f
// 
// Copyright September 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) with unit stride, test each of the 3 
//         possible types of complex blocks
//         V    a) VSIP block
//         US   b) User created split complex block
//         UI   c) User created interleaved complex block
//         VR   d) VSIP created real block
//         V  , VR -> US
//         US , VR -> UI
//      2) With various stride combinations, try
//         V  , VR -> US
//
// Revision History:
// September 4, 1999   Created
*/

#include <math.h>
#include "vsiptest_elwise.h"

char filename[] = "data/rcvmul_f";

int vttest_function (char *filename, 
                     vsip_cvview_f *view1, vsip_vview_f *view2, 
                     vsip_cvview_f *view3,
                     int length, int first_error)
{
  FILE *fp = fopen (filename, "r");
  char function_name[]="vsip_rcvmul_f()";
  int ret = 0;
  int read = 0;
  int val = 0;
  int i;
  vsip_scalar_f  val1, val2, val3, val4, val5;
  vsip_cscalar_f rval, cval1;
  vsip_scalar_f rrval, irval;

  if (!fp)
  {
    vtprintf ("Error in %s (%i)\n"
              "Could not open %s for reading\n", function_name,
              first_error+0, filename);
    ret |= 1<<(first_error+0);
    return (ret); 
  }

  val = vtgetlength (fp);

  if (val < length)
  {
    vtprintf ("Error in %s (%i)\n"
              "length (%i) in data file(%s)\n"
              "is shorter than views for testing(%i)\n", function_name,
              first_error+0, val, filename, length);
    ret |= 1<<(first_error+0);
    fclose (fp);
    return (ret); 
  }

  /* load the operand vectors */
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f %f,%f%*[^\n]\n", 
                   &val1, &val2, &val3);
    if (read<3)
    {
      vtprintf ("Error in %s (%i)\n"
                "Read %i elements in row %i of data file %s\n"
                "expecting 3\n", function_name, first_error+0, read, 
                i, filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    cval1 = vsip_cmplx_f (val2, val3);
    vsip_cvput_f (view1, i, cval1);
    vsip_vput_f (view2, i, val1);
  }
  fclose (fp);
  
  /* do the function */

  vsip_rcvmul_f (view2, view1, view3);

  /* check the results */

  fp = fopen (filename, "r");
  vtgetlength (fp);
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f %f,%f %f,%f\n", &val1, &val2, &val3, 
                   &val4, &val5);
    if (read<5)
    {
      vtprintf ("Error in %s (%i)\n"
                "Could not read result value for row %i of "
                "data file %s\n", function_name, first_error+0,
                filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    rval = vsip_cvget_f (view3, i);
    irval = vsip_imag_f (rval);
    rrval = vsip_real_f (rval);

    update_error_element (sqrt((rrval-val4)*(rrval-val4)+(irval-val5)*(irval-val5)));

    if ((fabs (rrval - val4) > VT_EM) || 
        (fabs (irval - val5) > VT_EM) )
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed closeness test - expected %f,%f "
                "got %f,%f\ninput=(%f:%f,%f)\n", function_name, 
                first_error+1, i, val4, val5, rrval, irval,
                vsip_vget_f (view2, i),
                vsip_real_f (vsip_cvget_f (view1, i)),
                vsip_imag_f (vsip_cvget_f (view1, i)));
      ret |= 1<<(first_error+1);
    }

    if ( (val4 <  0 && rrval >= 0) ||
         (val4 >  0 && rrval <= 0) ||
         (val4 != 0 && rrval == 0) ||
         (val5 <  0 && irval >= 0) ||
         (val5 >  0 && irval <= 0) ||
         (val5 != 0 && irval == 0) )
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed sign test - expected %f,%f got %f,%f\n",
                function_name, first_error+2, i, val4, val5,
                rrval, irval);
      ret |= 1<<(first_error+2);
    }

    if (ret)
    {
      fclose (fp);
      return (ret); 
    }
  }

  update_error_total ();

  fclose (fp);
  return (ret); 
}

int main(int argc, char *argv[])
{
  int length;
  int ret = 0;
  vsip_cvview_f *V, *US, *UI;
  vsip_vview_f *VR;
  vsip_cblock_f *csplitb, *cinterb;
  vsip_scalar_f *array1, *array2, *array3;
  int i;
  vsip_stride lstride;
  vsip_length templen;

  vsip_init ((void *)0);

  length = vtelwise_get_length (filename);
  if (!length)
  {
    vtprintf ("Error in %s (1)\n"
              "Could not get length from datafile %s\n",
              "vsip_rcvmul_f", filename);
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  V = vsip_cvcreate_f (length, 0);
  VR = vsip_vcreate_f (length, 0);

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*length);

  csplitb = vsip_cblockbind_f (array1, array2, length, 0);
  cinterb = vsip_cblockbind_f (array3, NULL, length,0);

  vsip_cblockadmit_f (csplitb, 0);
  vsip_cblockadmit_f (cinterb, 0);

  US = vsip_cvbind_f (csplitb, 0, 1, length);
  UI = vsip_cvbind_f (cinterb, 0, 1, length);

  error_init (argv[1]);
  if (!V)
  {
    vtprintf ("Error in %s (2)\n"
              "Could not allocate all test vectors\n", "vsip_rcvmul_f");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (ret |= vttest_function (filename,  V, VR, US, length, 3) )
  {
    vtprintf ("VSIPL created complex block testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("VSIPL created complex block test");

  if (ret |= vttest_function (filename, US, VR, UI, length, 6) )
  {
    vtprintf ("User created split complex block testing "
              "produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("User created split complex block test");

  vsip_cvalldestroy_f (UI);
  vsip_cvdestroy_f (US);
  vsip_valldestroy_f (VR);
  vsip_cvalldestroy_f (V);

  /* Stride tests */
  for (i=0; i<elements(VSIPTEST_ELWISE_LEFT_STRIDES); i++)
  {
    lstride = VSIPTEST_ELWISE_LEFT_STRIDES[i];
    
    templen = length / lstride;
    US = vsip_cvbind_f (csplitb, 0,      lstride, templen);
    VR = vsip_vcreate_f (templen, 0);
    V = vsip_cvcreate_f (templen, 0);

    if (ret |= vttest_function (filename, V, VR, US, templen, 9 ))
    {
      vtprintf ("Stride test (%i) produced errors\n", lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("Stride test (%i)", lstride);

    vsip_cvdestroy_f (US);
    vsip_valldestroy_f (VR);
    vsip_cvalldestroy_f (V);
  }
  error_finish();
  report (ret); 

  vsip_cblockdestroy_f (csplitb);
  free (array1);
  free (array2);
  free (array3);

  vsip_finalize ((void *)0);
  return (ret != 0);
}
