/* vsiptest_vdot_f.c Version 0.000 September 3, 1999
// 
// This file contains the source code included for testing 
// vsip_vdot_f
// 
// Copyright September 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) with unit strides, test each of the 8 possible types of
//         real blocks:
//         VR   a) VSIP real block
//         VRC  b) Derived from real part of VSIP complex block
//         VRI  c) Derived from imaginary part of VSIP complex block
//         UR   d) User real block
//         URCS e) Derived from real part of user split complex block
//         UICS f) Derived from imag part of user split complex block
//         URCI g) Derived from real part of user inter. complex block
//         UICI h) Derived from imag part of user inter. complex block
//              VR   , VRC
//              VRI  , UR
//              URCS , UICI
//              UICS , URCI
//      2) For various stride combinations,
//              UR , VR
//
// Revision History:
// September 2, 1999   Created
// March 20, 2002 [dpc] fixed some erroneous alldestroys
*/

#include <math.h>
#include "vsiptest_elwise.h"

int vttest_function (char *filename, 
                     vsip_vview_f *view1, vsip_vview_f *view2, 
                     int length, int first_error)
{
  FILE *fp = fopen (filename, "r");
  char function_name[] = "vsip_vdot_f";
  int ret = 0;
  int read = 0;
  int val = 0;
  int i;
  float total = 0;
  vsip_scalar_f rval, val1, val2;

  if (!fp)
  {
    vtprintf ("Error in %s (%i)\n"
              "Could not open %s for reading\n", function_name,
              first_error+0, filename);
    ret |= 1<<(first_error+0);
    return (ret); 
  }

  val = vtgetlength (fp);

  if (val < length)
  {
    vtprintf ("Error in %s (%i)\n"
              "length (%i) in data file(%s)\n"
              "is shorter than views for testing(%i)\n", function_name,
              first_error+0, val, filename, length);
    ret |= 1<<(first_error+0);
    fclose (fp);
    return (ret); 
  }

  /* load the operand vectors */
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f %f%*[^\n]\n", &val1, &val2);
    if (read<2)
    {
      vtprintf ("Error in %s (%i)\n"
                "Read %i elements in row %i of data file %s\n"
                "expecting 2\n", function_name, first_error+0, filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    vsip_vput_f (view1, i, val1); 
    vsip_vput_f (view2, i, val2);
    total += val1 * val2;
  }
  fclose (fp);
  
  /* do the function */

  rval = vsip_vdot_f (view1, view2);

  update_error_element (fabs(rval-total));
  /* check the results */

    if (fabs (rval - total) > VT_EM)
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed closeness test - expected %f "
                "got %f\ninput=(%f)\n", function_name, first_error+1, 
                i, total, rval, vsip_vget_f (view1, i));
      ret |= 1<<(first_error+1);
    }

    if ( (total <  0 && rval >= 0) ||
         (total >  0 && rval <= 0) ||
         (total != 0 && rval == 0) )
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed sign test - expected %f got %f\n",
                function_name, first_error+2, i, total, rval);
      ret |= 1<<(first_error+2);
    }

  update_error_total();
  return (ret); 
}


char filename[] = "data/vdot_f";

int main(int argc, char *argv[])
{
  int length;
  int ret = 0;
  vsip_vview_f *VR, *VR2, *VRC, *VRI, *UR, *UR2, *URCS, 
               *URCI, *UICS, *UICI;
  vsip_cvview_f *VC;
  vsip_block_f  *vblock, *ublock;
  vsip_cblock_f *csplitb, *cinterb;
  vsip_cvview_f *csplitv, *cinterv;
  vsip_scalar_f *array1, *array2, *array3, *array4;
  int i;
  vsip_stride lstride, rstride;
  vsip_offset offset;
  vsip_length templen;

  vsip_init ((void *)0);

  length = vtelwise_get_length (filename);
  if (!length)
  {
    vtprintf ("Error in %s (1)\n"
              "Could not get length from datafile %s\n",
              "vsip_vdot_f", filename);
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  VR = vsip_vcreate_f (length, 0);
  VC = vsip_cvcreate_f (length, 0);
  VRC = vsip_vrealview_f (VC);
  VRI = vsip_vimagview_f (VC);

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array4 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*length);

  vblock = vsip_blockcreate_f (length, 0);
  ublock = vsip_blockbind_f (array1, length, 0);
	vsip_blockadmit_f (ublock, 0);

  UR = vsip_vbind_f (ublock, 0, 1, length);
  csplitb = vsip_cblockbind_f (array2, array3, length, 0);
  cinterb = vsip_cblockbind_f (array4, NULL, length,0);
  vsip_cblockadmit_f (csplitb, 0);
  vsip_cblockadmit_f (cinterb, 0);

  csplitv = vsip_cvbind_f (csplitb, 0, 1, length);
  cinterv = vsip_cvbind_f (cinterb, 0, 1, length);

  URCS = vsip_vrealview_f (csplitv);
  URCI = vsip_vrealview_f (cinterv);
  UICS = vsip_vimagview_f (csplitv);
  UICI = vsip_vimagview_f (cinterv);

  error_init (argv[1]);

  if (!VR || !VRI || !VRC || !URCS || !URCI || !UICS || !UICI)
  {
    vtprintf ("Error in %s (2)\n"
              "Could not allocate all test vectors\n", "vsip_vdot_f");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (ret |= vttest_function (filename, VR, VRC, length, 3))
  {
    vtprintf ("VSIP created block into block derived from real portion "
              "of VSIP created\n"
              "complex block testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("VSIP created block into block derived from real portion "
                "of VSIP created\n complex block test");

  if (ret |= vttest_function (filename, VRI, UR, length, 6))
  {
    vtprintf ("Block derived from imaginary portion of VSIP created "
              "complex block\n"
              "into user created block testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("Block derived from imaginary portion of VSIP created "
                "complex block\n"
                "into user created block test");

  if (ret |= vttest_function (filename, URCS, UICI, length, 9))
  {
    vtprintf ("Block derived from real part of user created split "
              "complex block into\n"
              "block derived from imaginary part of user created "
              "interleaved complex block\n"
              "testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("Block derived from real part of user created split "
                "complex block into\n"
                "block derived from imaginary part of user created "
                "interleaved complex block test");

  if (ret |= vttest_function (filename, UICS, URCI, length, 12))
  {
    vtprintf ("Block derived from imaginary part of user created split "
              "complex block\n"
              "into block derived from real part of user created "
              "interleaved block\n"
              "testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("Block derived from imaginary part of user created split "
                "complex block\n"
                "into block derived from real part of user created "
                "interleaved block test");


  /* Stride tests */
  for (i=0; i<elements(VSIPTEST_ELWISE_LEFT_STRIDES); i++)
  {
    lstride = VSIPTEST_ELWISE_LEFT_STRIDES[i];
    rstride = VSIPTEST_ELWISE_RIGHT_STRIDES[i];
    if (rstride == 1)
    {
      offset = 0;
    }
    else if (rstride == -1)
    {
      offset = length-1;
    }
    else
    {
      vtprintf ("Illegal stride for right stride of stride test in\n"
                "Must be 1 or -1\n");
      report (-1); 
      vsip_finalize ((void *)0);
      return (-1 != 0);
    }
    templen = length / lstride;

    VR2 = vsip_vbind_f (vblock, 0, lstride, templen);
    UR2 = vsip_vbind_f (ublock, offset, rstride, templen);

    if (ret |= vttest_function (filename, VR2, UR2, templen, 15) )
    {
      vtprintf ("Stride test (%i)->%i produced errors\n", lstride,
                rstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("Stride test (%i)->%i", lstride, rstride);

    vsip_vdestroy_f (VR2);
    vsip_vdestroy_f (UR2);
  }
  vsip_blockdestroy_f (vblock);

  error_finish();
  report (ret); 

  vsip_vdestroy_f (URCS);
  vsip_vdestroy_f (URCI);
  vsip_vdestroy_f (UICS);
  vsip_vdestroy_f (UICI);
  vsip_valldestroy_f (UR);
  vsip_valldestroy_f (VR);
  vsip_vdestroy_f (VRC);
  vsip_vdestroy_f (VRI);

  vsip_cvalldestroy_f (csplitv);
  vsip_cvalldestroy_f (cinterv);
  vsip_cvalldestroy_f (VC);

  free (array1);
  free (array2);
  free (array3);
  free (array4);


  vsip_finalize ((void *)0);
  return (ret != 0);
}
