/* vsiptest_firflt_f.c Version 0.000 November 8, 1999
// 
// This file contains the vsiptest_firflt_f test functionality.
// 
// Functional Description:
//   vsip_firflt_f does a FIR filter and decimates the output
//  
//
// Copyright November 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Revision History:
// Nov  8 1999 - Created
*/

#include"vsiptest_sigproc.h"
#include <stdio.h>

void vector_read (char *filename, vsip_vview_f **input, 
                  vsip_vview_f **output, vsip_vview_f **kernel, 
                  vsip_symmetry *symm, vsip_length *N, vsip_length *D, 
                  vsip_obj_state *state, vsip_length *ntimes)
{
  FILE *fp;
  int read = 0;
  int i;
  float val1, val2, val3;
  int n, d, nt, kernel_size, sym;
  int s;

  fp = fopen (filename, "r");
  if (!fp)
  {
    vtprintf ("Error in vsip_firflt_f (0)\n"
              "unable to open data file %s for reading\n", filename);
    exit(1);
  }

  read = fscanf (fp, "%i %i %i %i %i %i\n", &sym, &n, &d, &s, &nt, &kernel_size);
  *D = d;
  *ntimes = nt;
  *N = n;
  *symm = sym;
  *state = s;

  if (read != 6)
  {
    vtprintf ("Error in vsip_firflt_f (0)\n"
              "unable to read FIR parameters in %s\n", filename);
    fclose (fp);
    exit (1);
  }
  
  *input = vsip_vcreate_f (n, 0);
  *kernel = vsip_vcreate_f (kernel_size, 0);
  *output = vsip_vcreate_f (n, 0);

  if (!input || !output || !kernel)
  {
    vtprintf ("Error in vsip_firflt_f (0)\n"
              "unable to create views of size %i for %s\n", n, filename);
    fclose (fp);
    exit(1);
  }

  for (i=0; i<n; i++)
  {
    read = fscanf (fp, "%f %f %f\n", &val1, &val2, &val3);
    if (read < 3)
    {
      vtprintf ("Error in vsip_firflt_f (0)\n"
                "unable to read row %i in %s\n", i, filename);
      fclose (fp);
      exit (1);
    }
    vsip_vput_f (*input, i, val1);
    vsip_vput_f (*output, i, val2);
    if (i < kernel_size)
    {
      vsip_vput_f (*kernel, i, val3);
    }
  }
  fclose (fp);
}

int main(int argc, char *argv[])
{
  int ret = 0;
  int numfiles = VSIPTEST_FIRFLT_F_NUMFILES;
  char filename[120];
  int trial;
  vsip_vview_f *tempv, *outputv;
  vsip_vview_f *inputv, *kernelv;
  vsip_length N, D, ntimes;
  vsip_symmetry symmetry;
  int i;
  vsip_scalar_f e_real, r_real;
  vsip_obj_state state;
  vsip_fir_f *fir = NULL;
  int max;
  vsip_obj_state prev_state = VSIP_STATE_NO_SAVE;


  /* enum tests */
  vsip_symmetry st;
  st = VSIP_NONSYM;
  st = VSIP_SYM_EVEN_LEN_ODD;
  st = VSIP_SYM_EVEN_LEN_EVEN;
  st = st;

  vsip_init ((void *)0);

  error_init (argv[1]);

  for (trial=0; trial<numfiles; trial++)
  {
    sprintf (filename, "data/%s%i", VSIPTEST_FIRFLT_F_BASENAME, 
             trial+1);
    vector_read (filename, &inputv, &outputv, &kernelv, &symmetry, &N, &D, 
                &state, &ntimes);

    tempv = vsip_vcreate_f (ceil((float)N/(float)D), 0);
    
    if (prev_state != VSIP_STATE_SAVE)
    {
      fir = vsip_fir_create_f (kernelv, symmetry, N, D, state, ntimes, 0);
    }
    prev_state = state;

    if (!fir)
    {
      vtprintf ("Error in vsip_fir_create_f (2)\n"
                "Could not create FIR object of N=%i D=%i "
                "ntimes=%i\n", N, D, ntimes);
      ret |= 1<<2;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    vsip_firflt_f (fir, inputv, tempv);

    max = ceil ((float)N/(float)D);

    for (i=0; i<max; i++)
    {
      r_real = vsip_vget_f (tempv, i);

      e_real = vsip_vget_f (outputv, i);

      update_error_element (fabs(r_real - e_real));
      if (fabs (r_real - e_real) > VSIPTEST_FIRFLT_F_MAX_EPSILON)
      {
        vtprintf ("Error in vsip_firflt_f (3)\n"
                  "Element %i of result vector (%f) did not match "
                  "expected value (%f)\n"
                  "for N=%i D=%i ntimes=%i trial=%i\n", i, r_real, e_real, 
                   N, D, ntimes, trial+1);
        ret |= 1<<3;
        break;
      }
      if ((r_real < 0 && e_real >= 0) || 
          (r_real == 0 && e_real != 0) ||
          (r_real > 0 && e_real <= 0) )
      {
        vtprintf ("Error in vsip_firflt_f (4)\n"
                  "Element %i of result vector (%f) failed sign test "
                  "against expected value (%f)\n"
                  "for N=%i D=%i ntimes=%i trial=%i\n", i, r_real,
                  e_real, N, D, ntimes, trial+1);
        ret |= 1<<4;
        break;
      }
    }
    update_error_total();
    error_report_test ("Test #%i for N=%i D=%i ntimes=%i", trial+1, N, D, ntimes);
    if (state == VSIP_STATE_NO_SAVE)
    {
      vsip_fir_destroy_f (fir);
      fir = NULL;
    }
    vsip_valldestroy_f (tempv);
    vsip_valldestroy_f (kernelv);
    vsip_valldestroy_f (inputv);
    vsip_valldestroy_f (outputv);
  }
  if (fir != NULL)
  {
    vsip_fir_destroy_f (fir);
  }
  error_finish();
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
