/* vsiptest_cvsubview_f.c Version 0.000 June 17, 1999
// 
// This file contains the vsiptest_vcsubview_f test functionality.
// 
// Functional Description:
//   vsip_cvsubview_f creates a subset of a view
//
// Copyright May 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Create a large block and view.
//   2) Fill the view with ramp data
//   3) for a variety of strides, offsets, and lengths:
//      a) create a subview
//      b) verify the subview data
//      c) fill the subview with different data
//      d) verify the master view data is correct
//      e) reset master view data
//   4) Create a subview of the master view
//   5) Create a subview of the subview
//   6) Verify that sub-subview attribs are correct, and data is as expected
//
// Revision History:
// May 25, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_cvview_f *view;
  vsip_cblock_f *block;
  int            have_malloc_failer = vthas_fail_mech();
  int            ret = 0;
  vsip_cscalar_f cval;
  vsip_index     index, index1;
  vsip_length    length;
  vsip_cvattr_f  attrib;
  vsip_cvview_f *sview, *ssview;
  vsip_cscalar_f dval;

  vsip_init ((void *)0);

  block = vsip_cblockcreate_f (VSIPTEST_CVSUBVIEW_F_BLOCK_SIZE, 0);
  view  = vsip_cvbind_f (block, 0, 1, VSIPTEST_CVSUBVIEW_F_BLOCK_SIZE);
  for (i=0; i<VSIPTEST_CVSUBVIEW_F_BLOCK_SIZE; i++)
  {
    cval.r = i;
    cval.i = i * 3;
    vsip_cvput_f (view, i, cval);
  }

  if (have_malloc_failer)
  {
    vtfail_next_malloc(1);
    sview = vsip_cvsubview_f (view, 0, 1);
    if (sview != NULL)
    {
        vtprintf ("Error in vsip_cvsubview()_f (1)\n"
                  "Failed subview did not return NULL\n");
        ret |= 1<<1;
    }
    vsip_cvdestroy_f (sview);
  }

  for (i=0; i<elements(VSIPTEST_CVSUBVIEW_F_LENGTH); i++)
  {
    index  = VSIPTEST_CVSUBVIEW_F_INDEX[i];
    length = VSIPTEST_CVSUBVIEW_F_LENGTH[i];


    for (i=0; i<VSIPTEST_CVSUBVIEW_F_BLOCK_SIZE; i++)
    {
      cval.r = i;
      cval.i = i * 3;
      vsip_cvput_f (view, i, cval);
    }

    sview = vsip_cvsubview_f (view, index, length);

    /* Verify subview data */
    for (j=0; j<length; j++)
    {
      cval = vsip_cvget_f (sview, j);
      if (vtfcomp (cval.r, index+j) ||
          vtfcomp (cval.i, (index+j)*3) )
      {
        vtprintf ("Error in vsip_cvsubview()_f (2)\n"
                  "Data element %i did not match master view for index:%i "
                  "length:%i\n", j, index, length);
        ret |= 1<<2;
        break;
      }
    }

    /* ramp, then verify master view */
    for (j=0; j<length; j++)
    {
      cval.r = j * 5;
      cval.i = j * 7;
      vsip_cvput_f (sview, j, cval);
    }

    for (j=0; j<VSIPTEST_CVSUBVIEW_F_BLOCK_SIZE; j++)
    {
      if (j<index || j>=index+length)
      {
        dval.r = j;
        dval.i = j * 3;
      }
      else
      {
        dval.r = 5 * (j-index);
        dval.i = 7 * (j-index);
      }
      cval = vsip_cvget_f (view, j);

      if (vtfcomp (cval.r, dval.r) ||
          vtfcomp (cval.i, dval.i) )
      {
        vtprintf ("Error in vsip_cvsubview_f() (3)\n"
                  "Data element %i in master view did not match expected for "
                  "index:%i length:%i\n(%f %f)(%f %f)\n", j, index, length,
                  cval.r, cval.i, dval.r, dval.i);
        ret |= 1<<3;
        break;
      }
    }
    vsip_cvgetattrib_f (sview, &attrib);
    if (attrib.length != length ||
        attrib.stride != 1      ||
        attrib.offset != index)
    {
      vtprintf ("Error in vsip_cvsubview_f() (4)\n"
                "Subview with index:%i length:%i did not have correct "
                "attributes\n", index, length);
      ret |= 1<<4;
    }
    vsip_cvdestroy_f (sview);
  }

  for (i=0; i<VSIPTEST_CVSUBVIEW_F_BLOCK_SIZE; i++)
  {
    cval.r = i;
    cval.i = i * 3;
    vsip_cvput_f (view, i, cval);
  }

  index = VSIPTEST_CVSUBVIEW_F_SUBINDEX;
  length = VSIPTEST_CVSUBVIEW_F_BLOCK_SIZE - index;

  sview = vsip_cvsubview_f (view, VSIPTEST_CVSUBVIEW_F_SUBINDEX, length);

  index1= length/2;
  length /= 2;
  ssview = vsip_cvsubview_f (sview, index1, length);

  if (!ssview)
  {
    vtprintf ("Error in vsip_cvsubview_f() (5)\n"
              "Could not create subview of subview\n");
    ret |= 1<<5;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  
  vsip_cvgetattrib_f (ssview, &attrib);
  if (attrib.length != length ||
      attrib.stride != 1      ||
      attrib.offset != index+index1)
  {
    vtprintf ("Error in vsip_cvsubview_f() (6)\n"
              "Subview of subview did not have correct attributes\n"
              "Expected o:%i s:%i l:%i, found o:%i s:%i l:%i\n",
              index+index1, 1, length, attrib.offset, attrib.stride, 
              attrib.length);
    ret |= 1<<6;
  }

  /* Verify subview data */
  for (j=0; j<length; j++)
  {
    cval = vsip_cvget_f (ssview, j);
    if (vtfcomp (cval.r, (index1+index+j)) ||
        vtfcomp (cval.i, (index1+index+j)*3) )
    {
      vtprintf ("Error in vsip_cvsubview()_f (7)\n"
                "Data element %i of subview of subview did not match master "
                "view\n", j);
      ret |= 1<<7;
      break;
    }
  }

  /* ramp, then verify master view */
  for (j=0; j<length; j++)
  {
    cval.r = j * 5;
    cval.i = j * 7;
    vsip_cvput_f (ssview, j, cval);
  }
  for (j=0; j<VSIPTEST_CVSUBVIEW_F_BLOCK_SIZE; j++)
  {
    if (j<index+index1 || j>=index+index1+length)
    {
      dval.r = j;
      dval.i = j * 3;
    }
    else
    {
      dval.r = 5 * (j-index-index1);
      dval.i = 7 * (j-index-index1);
    }
    cval = vsip_cvget_f (view, j);
    if ((vtfcomp (cval.r, dval.r)) ||
        (vtfcomp (cval.i, dval.i)) )
    {
      vtprintf ("Error in vsip_cvsubview_f() (8)\n"
                "Data element %i in master view did not match expected for "
                "subview of subview\n"
                "Expected: %f, got: %f\n", j, dval.r, cval.r);
      ret |= 1<<8;
      break;
    }
  }
  vsip_cvdestroy_f (ssview);
  vsip_cvdestroy_f (sview);
  vsip_cvdestroy_f (view);
  vsip_cblockdestroy_f (block);

  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
