/* vsiptest_vrealview_f.c Version 0.000 May 25, 1999
// 
// This file contains the vsiptest_vrealview_f test functionality.
// 
// Functional Description:
//   vsip_vrealview_f creates a derived block object from the real part of 
//   a complex view (and thus, complex object), and returns a view to the 
//   derived object
//
// Copyright May 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Test that a memory allocation error causes cloneview to return NULL 
//   2) Create split and interleaved blocks filled with data
//   3) for various strides, offsets, lengths, for each test block:
//      a) bind a complex vector to the block
//      b) get the realview
//      c) verify the attributes are as requested
//      d) read back and verify data
//      e) verify that the derived block acts like a block
//
// Revision History:
// May 25, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_cblock_f *sblock, *iblock;
  vsip_vview_f  *view;
  vsip_cvview_f *cview = NULL;
  vsip_vview_f  *rview;
  int            have_malloc_failer = vthas_fail_mech();
  int            ret = 0;
  vsip_scalar_f  val;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_vattr_f   attrib;
  vsip_cvview_f *sview, *iview;
  vsip_scalar_f *array1, *array2, *array3, *array4;
  vsip_block_f  *dblock;
  int            result;

  vsip_init ((void *)0);

  array1 = (vsip_scalar_f *)
            malloc(VSIPTEST_VREALVIEW_F_BLOCK_SIZE*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)
            malloc(VSIPTEST_VREALVIEW_F_BLOCK_SIZE*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)
            malloc(VSIPTEST_VREALVIEW_F_BLOCK_SIZE*sizeof(vsip_scalar_f)*2);

  if (!array1 || !array2 || !array3)
  {
   
    vtprintf ("Error in vsip_vrealview_f() (1)\n"
              "Could not allocate split and interleaved data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }


  sblock = vsip_cblockbind_f (array1, array2, 
                              VSIPTEST_VREALVIEW_F_BLOCK_SIZE, 0);
  iblock = vsip_cblockbind_f (array3, NULL, 
                              VSIPTEST_VREALVIEW_F_BLOCK_SIZE, 0);
  
  for (i=0; i<VSIPTEST_VREALVIEW_F_BLOCK_SIZE; i++)
  {
    array1[i]     = (vsip_scalar_f) i;
    array2[i]     = (vsip_scalar_f) i * 2;
    array3[i*2]   = (vsip_scalar_f) i * 4;
    array3[i*2+1] = (vsip_scalar_f) i * 8;
  }
  vsip_cblockadmit_f (sblock, 1);
  vsip_cblockadmit_f (iblock, 1);

  if (!iblock || !sblock)
  {
    vtprintf ("Error in vsip_vrealview_f() (2)\n"
              "Could not create interleaved and split blocks of length %i\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (have_malloc_failer)
  {
    cview = vsip_cvbind_f (iblock, 0, 1, 1);
    vtfail_next_malloc(1);
    rview = vsip_vrealview_f (cview);
    if (rview != NULL)
    {
        vtprintf ("Error in vsip_vrealview() (3)\n"
                  "Failed call did not return NULL\n");
        ret |= 1<<3;
    }
    vsip_vdestroy_f (rview);
    vsip_cvdestroy_f (cview);
  }

  for (i=0; i<elements(VSIPTEST_VREALVIEW_F_LENGTH); i++)
  {
    vsip_cblockrelease_f (sblock, 0, &array1, &array2);
    vsip_cblockrelease_f (iblock, 0, &array3, &array4);
    for (j=0; j<VSIPTEST_VREALVIEW_F_BLOCK_SIZE; j++)
    {
      array1[j]     = (vsip_scalar_f) j;
      array2[j]     = (vsip_scalar_f) j * 2;
      array3[j*2]   = (vsip_scalar_f) j * 4;
      array3[j*2+1] = (vsip_scalar_f) j * 8;
    }
    vsip_cblockadmit_f (sblock, 1);
    vsip_cblockadmit_f (iblock, 1);
   
    offset = VSIPTEST_VREALVIEW_F_OFFSET[i];
    stride = VSIPTEST_VREALVIEW_F_STRIDE[i];
    length = VSIPTEST_VREALVIEW_F_LENGTH[i];

    iview = vsip_cvbind_f (iblock, offset, stride, length);
    sview = vsip_cvbind_f (sblock, offset, stride, length);

    if (iview == NULL)
    {
      vtprintf ("Error in vsip_vrealview_f() (4)\n"
                "Failed to bind view(o:%i s:%i l:%i) to interleaved block \n",
                offset, stride, length);
      ret |= 1<<4;
    }
    if (sview == NULL)
    {
      vtprintf ("Error in vsip_vrealview_f() (5)\n"
                "Failed to bind view(o:%i s:%i l:%i) to split block \n",
                offset, stride, length);
      ret |= 1<<5;
    }

    /* split first */

    view = vsip_vrealview_f (sview);

    /* check the values */
    for (j=0; j<length; j++)
    {
      val = vsip_vget_f (view, j);
      if (vtfcomp (val, (offset + j * stride)))
      {
        vtprintf ("Error in vsip_vrealview_f (6)\n"
                  "Data element %i in Real view(o:%i s:%i l:%i) to split block "
                  "did not match array\n", j, stride, offset, length);
        ret |= 1<<6;
        break;
      }
    }

    /* check the attribs */
    vsip_vgetattrib_f (view, &attrib);
    if (attrib.length != length )
    {
      vtprintf ("Error in vsip_vrealview_f (7)\n"
                "Real view vector to split block did not match expected "
                "attributes\nExpected o:%i s:%i l:%i, found o:%i s:%i l:%i\n",
                offset, stride, length, attrib.offset, attrib.stride,
                attrib.length);
      ret |= 1<<7;
    }

    /* check the derived block */
    dblock = vsip_vgetblock_f (view);

    result = vtcheck_f_block (dblock, length);
    if (result)
    {
      vtprintf ("Error in vsip_vrealview_f (8)\n"
                "Derived block from real view of split block did not behave "
                "as a block:%i (o:%i s:%i l:%i)\n", result, offset, stride, 
                length);
      ret |= 1<<8;
    }

    /* now interelaved.... */

    view = vsip_vrealview_f (iview);

    /* check the values */
    for (j=0; j<length; j++)
    {
      val = vsip_vget_f (view, j);
      if (vtfcomp (val, (offset + j * stride) * 4))
      {
        vtprintf ("Error in vsip_vrealview_f (9)\n"
                  "Data element %i in Real view(o:%i s:%i l:%i) to interleaved "
                  "block did not match array\n", j, stride, offset, length);
        ret |= 1<<9;
        break;
      }
    }

    /* check the attribs */
    vsip_vgetattrib_f (view, &attrib);
    if (attrib.length != length)
    {
      vtprintf ("Error in vsip_vrealview_f (10)\n"
                "Real view vector to interleaved block did not match expected "
                "attributes\nExpected o:%i s:%i l:%i, found o:%i s:%i l:%i\n",
                offset, stride, length, attrib.offset, attrib.stride,
                attrib.length);
      ret |= 1<<10;
    }

    /* check the derived block */
    dblock = vsip_vgetblock_f (view);

    result = vtcheck_f_block (dblock, length);
    if (result)
    {
      vtprintf ("Error in vsip_vrealview_f (11)\n"
                "Derived block from real view of interleaved block did not "
                "behave as a block:%i (o:%i s:%i l:%i)\n",
                 result, offset, stride, length);
      ret |= 1<<11;
    }
  }
  vsip_cvdestroy_f (iview);
  vsip_cvdestroy_f (sview);
  vsip_cblockdestroy_f (iblock);
  vsip_cblockdestroy_f (sblock);

  report (ret); 
    vsip_finalize ((void *)0);
  return (ret != 0);
}
