/* vsiptest_vsubview_f.c Version 0.000 May 25, 1999
// 
// This file contains the vsiptest_vsubview_f test functionality.
// 
// Functional Description:
//   vsip_vsubview_f creates a subset of a view
//
// Copyright May 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Create a large block and view.
//   2) Fill the view with ramp data
//   3) for a variety of strides, offsets, and lengths:
//      a) create a subview
//      b) verify the subview data
//      c) fill the subview with different data
//      d) verify the master view data is correct
//      e) reset master view data
//   4) Create a subview of the master view
//   5) Create a subview of the subview
//   6) Verify that sub-subview attribs are correct, and data is as expected
//
// Revision History:
// May 25, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_vview_f  *view;
  vsip_block_f  *block;
  int            have_malloc_failer = vthas_fail_mech();
  int            ret = 0;
  vsip_scalar_f  val;
  vsip_index     index, index1;
  vsip_length    length;
  vsip_vattr_f   attrib;
  vsip_vview_f  *sview, *ssview;
  vsip_scalar_f  dval;

  vsip_init ((void *)0);

  block = vsip_blockcreate_f (VSIPTEST_VSUBVIEW_F_BLOCK_SIZE, 0);
  view  = vsip_vbind_f (block, 0, 1, VSIPTEST_VSUBVIEW_F_BLOCK_SIZE);
  vsip_vramp_f (0.0, 1.0, view);

  if (have_malloc_failer)
  {
    vtfail_next_malloc(1);
    sview = vsip_vsubview_f (view, 0, 1);
    if (sview != NULL)
    {
        vtprintf ("Error in vsip_vsubview()_f (1)\n"
                  "Failed subview did not return NULL\n");
        ret |= 1<<1;
    }
    vsip_vdestroy_f (sview);
  }

  for (i=0; i<elements(VSIPTEST_VSUBVIEW_F_LENGTH); i++)
  {
    index  = VSIPTEST_VSUBVIEW_F_INDEX[i];
    length = VSIPTEST_VSUBVIEW_F_LENGTH[i];

    sview = vsip_vsubview_f (view, index, length);

    /* Verify subview data */
    for (j=0; j<length; j++)
    {
      val = vsip_vget_f (sview, j);
      if (vtfcomp (val, index+j))
      {
        vtprintf ("Error in vsip_vsubview()_f (2)\n"
                  "Data element %i did not match master view for index:%i "
                  "length:%i\n", j, index, length);
        ret |= 1<<2;
        break;
      }
    }

    /* ramp, then verify master view */
    vsip_vramp_f (4.0, 0.5, sview);
    for (j=0; j<VSIPTEST_VSUBVIEW_F_BLOCK_SIZE; j++)
    {
      if (j<index || j>=index+length)
      {
        dval = j;
      }
      else
      {
        dval = 4.0 + 0.5 * (j-index);
      }

      if (vtfcomp (vsip_vget_f(view, j), dval))
      {
        vtprintf ("Error in vsip_vsubview_f() (3)\n"
                  "Data element %i in master view did not match expected for "
                  "index:%i length:%i\n", j, index, length);
        ret |= 1<<3;
        break;
      }
    }
    vsip_vgetattrib_f (sview, &attrib);
    if (attrib.length != length ||
        attrib.stride != 1      ||
        attrib.offset != index)
    {
      vtprintf ("Error in vsip_vsubview_f() (4)\n"
                "Subview with index:%i length:%i did not have correct "
                "attributes\n", index, length);
      ret |= 1<<4;
    }
    vsip_vdestroy_f (sview);
    vsip_vramp_f (0.0, 1.0, view);
  }

  index = VSIPTEST_VSUBVIEW_F_SUBINDEX;
  length = VSIPTEST_VSUBVIEW_F_BLOCK_SIZE - index;

  sview = vsip_vsubview_f (view, VSIPTEST_VSUBVIEW_F_SUBINDEX, length);

  index1= length/2;
  length /= 2;
  ssview = vsip_vsubview_f (sview, index1, length);

  if (!ssview)
  {
    vtprintf ("Error in vsip_vsubview_f() (5)\n"
              "Could not create subview of subview\n");
    ret |= 1<<5;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_vgetattrib_f (ssview, &attrib);
  if (attrib.length != length ||
      attrib.stride != 1      ||
      attrib.offset != index+index1)
  {
    vtprintf ("Error in vsip_vsubview_f() (6)\n"
              "Subview of subview did not have correct attributes\n"
              "Expected o:%i s:%i l:%i, found o:%i s:%i l:%i\n",
              index+index1, 1, length, attrib.offset, attrib.stride, 
              attrib.length);
    ret |= 1<<6;
  }

  /* Verify subview data */
  for (j=0; j<length; j++)
  {
    val = vsip_vget_f (ssview, j);
    if (vtfcomp (val, index1+index+j))
    {
      vtprintf ("Error in vsip_vsubview()_f (7)\n"
                "Data element %i of subview of subview did not match master "
                "view\n", j);
      ret |= 1<<7;
      break;
    }
  }

  /* ramp, then verify master view */
  vsip_vramp_f (4.0, 0.5, ssview);
  for (j=0; j<VSIPTEST_VSUBVIEW_F_BLOCK_SIZE; j++)
  {
    if (j<index+index1 || j>=index+index1+length)
    {
      dval = j;
    }
    else
    {
      dval = 4.0 + 0.5 * (j-index-index1);
    }
    if (vtfcomp (vsip_vget_f(view, j), dval))
    {
      vtprintf ("Error in vsip_vsubview_f() (8)\n"
                "Data element %i in master view did not match expected for "
                "subview of subview\n"
                "Expected: %f, got: %f\n", j, dval, vsip_vget_f(view, j));
      ret |= 1<<8;
      break;
    }
  }
  vsip_vdestroy_f (ssview);
  vsip_vdestroy_f (sview);
  vsip_vdestroy_f (view);
  vsip_blockdestroy_f (block);

  report (ret); 
    vsip_finalize ((void *)0);
  return (ret != 0);
}
