/* vsiptest_blockbind_f.c Version 0.000 May 5, 1997
// 
// This file contains the vsiptest_blockbind_f test functionality.
//
// Functional Description:
//   vsip_blockbind_f binds to a public data array with at least N
//   vsip_scalar_f real elements.
//
// Copyright May 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Thomas Egolf
//         egolf@ece.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//      various sizes using the malloc function call and then bind the
//      public block to a private block using this function. This test is
//      similar to the block create functionality however in this case the
//      block is created by the application developer.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements after binding
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for binding for negative input lengths (error conditions)
//
// VSIP functions called: vsip_blockbind_f, vsip_vbind_f, vsip_vget_f,
//                        vsip_vgetlength_f, vsip_vdestroy_f
//
// Revision History:
// May 6, 1997  Added copyright notice.             
// May 6, 1997  Executed tests 1-4 without error and block sizes up to 5000 */

#include<stdio.h>
#include<stdlib.h>  /* For malloc functionality */
#include "vsiptest_block.h"

int main ()
{
  size_t             block_size, i, j;
  vsip_scalar_f     *public_block;
  vsip_memory_hint   hint = 0;
  vsip_block_f      *bind_block;
  vsip_vview_f      *blockview;
  vsip_scalar_f      minval, maxval;
  vsip_scalar_vi     index1, index2;
  vsip_length        length;
  vsip_vattr_f       attrib;
  vsip_bool          vb;
  vsip_memory_hint   vh;

  int ret = 0;

  public_block = NULL;

  /* API Checks */
  vb = VSIP_TRUE;
  vb = VSIP_FALSE;
  vb = vb;

  vh = VSIP_MEM_NONE;
  vh = VSIP_MEM_RDONLY;
  vh = VSIP_MEM_CONST;
  vh = VSIP_MEM_SHARED;
  vh = VSIP_MEM_SHARED_RDONLY;
  vh = VSIP_MEM_SHARED_CONST;
  vh = vh;

  vsip_init ((void *)0);
  bind_block = vsip_blockbind_f (public_block, (vsip_length)(1), hint);
  if (bind_block == NULL)
  {
    vtprintf("Error in vsip_blockbind_f(); (1) \n"
             "block bind to NULL pointer returns failure\n");
    ret |= 1<<1;
  }

  /* Check various public block sizes from 2 to block_length_max */
  for (j=0; j<elements (VSIPTEST_BLOCKBIND_SIZE_ARRAY); j++)
  {
    block_size = VSIPTEST_BLOCKBIND_SIZE_ARRAY[j];
    public_block = (float *)malloc(block_size*sizeof(float));

    /* Fill the block with data values equal to its index value */
    for(i=0; i<block_size; i++) 
    {
      *(public_block + i) = (float)i;
    }

    /********************* VSIP_BLOCKBIND_D **************************
      Bind to the public block with at least block_size real
      elements and look at the contents of the binded block. */

    bind_block = vsip_blockbind_f(public_block, block_size, hint);
    vsip_blockadmit_f (bind_block, 1);

    /****************************************************************/

    /********************* VSIP_VBIND_D *****************************
      Create a view on the bound block */

    blockview = vsip_vbind_f(bind_block, 0, 1, (vsip_length)block_size);

    /* Check some of the parameters of this function
    //    1) Check the first and last array elements
    //    2) Check the size of the block */

    /********************* VSIP_VGET_D *******************************
      Check the minimum and maximum values of the view on the block */

    index1 = 0; index2 = block_size-1;
    minval = vsip_vget_f(blockview, index1);
    maxval = vsip_vget_f(blockview, index2);

    if(vtfcomp (minval, (vsip_scalar_f)0.0))
    {
          vtprintf("Error in vsip_blockbind_f(); (2) \n"
       "First value assignment does not match first element value \n");
          ret |= 1<<2;
    }

    if(vtfcomp (maxval, (vsip_scalar_f)(block_size-1)))
    {
      vtprintf("Error in vsip_blockbind_f(); (3) \n"
               "Last value assignment does not match last element value \n");
      ret |= 1<<3;
    }

    /********************* VSIP_VGETLENGTH_D ************************
      Check the length of the block allocated */

    vsip_vgetattrib_f(blockview, &attrib);
    length = attrib.length;
    if(length != (vsip_length)block_size)
    {
      vtprintf("Error in vsip_blockbind(); (4) \n"
       "Size does not match allocated size \n");
      ret |= 1<<4;
    }

    /********************* VSIP_VDESTROY_D ***************************
      Destroy the view on the block */
    vsip_vdestroy_f(blockview);

    free(public_block);
  }
  report (ret);
  vsip_finalize ((void *)0);
  return (ret != 0);
}
