/* vsiptest_blockdestroy_i.c Version 0.000 May 6, 1997
// 
// This file contains the vsiptest_blockdestroy_i test functionality.
//
// Functional Description:
//   vsip_blockdestroy_i destroys the pointer (vsip_block_i* b) to the
//   data array if the data array is private and then returns
//   a null pointer. If the data array is public, it returns a pointer to
//   the public data array (vsip_block_i* b)
//
// Copyright May 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Thomas Egolf
//         egolf@ece.gatech.edu
//         Dan Campbell [adapted to _i]
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real public blocks of data
//      with various sizes using the malloc function call and then bind the
//      public block to a private block using the function vsip_blockbind_i.
//      Also a set of tests will create private blocks using the
//      vsip_blockcreate_i and vsip_cblockcreate_i functions and test
//      whether the value returned by vsip_blockfind_i is null. Another set
//      of tests verifies that when there are multiple bindings to a private
//      object, the vsip_blockdestroy_i function will destroy the object
//      even if all bindings are not removed. In this case the user has the
//      option of setting the number of views to create and destroy using
//      the VSIP function vsip_vbind_i.c and vsip_vdestroy_i.c. If the
//      number of views created is greater than the number of views destroyed
//      then the vsip_blockdestroy_i function will warn the user if the
//      VSIP_DEVELOPMENT_MODE switch is turned on. It will notify the user how
//      many bindings are still attached to the block object.
//
//      Check some of the parameters of this function
//        1) Verify that when a public array is bound to a private area
//           and destroyed by the vsip_blockdestroy_i function, the returned
//           value is a pointer to the public data array. In this test we
//           check the pointer to the array that was returned from the
//           vsip_blockdestroy_i function and verify that the data is the
//           same as initially used to fill the array.
//        2) Verify that when a private block of data is destroyed using
//           vsip_blockdestroy_i, the returned value of the function is a
//           null pointer.
//        3) Verify that when there are multiple binding to a private object,
//           a call to vsip_blockdestroy_i does destroy the object even if
//           all the bindings are not removed.
//
// VSIP functions called: vsip_blockdestroy_i, vsip_blockcreate_i,
//                        vsip_cblockcreate_i, vsip_blockbind_i,
//                        vsip_vbind_i.c, vsip_vdestroy_i.c
//                        
//
// Revision History:
// May   6, 1997  Added copyright notice.             
// May   7, 1997  Executed tests 1-3 without error and block sizes up to 5000 
// April 7, 1999 Adapted to _i.
*/

#include<stdio.h>
#include<stdlib.h>  /* For malloc functionality */
#include "vsiptest_block.h"

int main()
{

  size_t            block_size, i;
  vsip_memory_hint  hint = 0;
  vsip_scalar_i     *public_block;
  vsip_block_i      *bind_block;
  int               j;

  vsip_init ((void *)0);
  /* Check various public block sizes from 1 to block_length_max */
  for (j=0; j<elements (VSIPTEST_BLOCKDESTROY_I_BLOCK_SIZE); j++)
  {
    block_size = VSIPTEST_BLOCKDESTROY_I_BLOCK_SIZE[j];

    public_block = (vsip_scalar_i*)malloc(block_size*sizeof(vsip_scalar_i));

    /* Fill the block with data values equal to its index value */
    for(i=0; i<block_size; i++) 
    {
      *(public_block + i) = (vsip_scalar_i)i;
    }

    /********************* VSIP FUNCTION CALL ***********************
      Bind to the public block with at least block_size vsip_scalar_i
      real elements and look at the contents of the binded block. */
    bind_block = vsip_blockbind_i(public_block, block_size, hint);
    /****************************************************************/

    vsip_blockdestroy_i(bind_block);

    bind_block = vsip_blockbind_i (public_block, block_size, hint);
    vsip_blockadmit_i (bind_block, 1);
    vsip_blockdestroy_i (bind_block);
  }
  report (0);
  vsip_finalize ((void *)0);
  return (0);
}
