/* vsiptest_blockfind_f.c Version 0.000 April 22, 1999
// 
// This file contains the vsiptest_cblockfind_f test functionality.
//
// Functional Description:
//   vsip_cblockfind_f returns a pointer to the public data arrays
//   if vsip_block_f *b is public, otherwise it returns NULL 
//   if the data is in complex interleaved mode, pointer2 is NULL
//
// Copyright April 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Thomas Egolf
//         egolf@ece.gatech.edu
//         Dan Campbell [adapted to c*_f]
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//      various sizes using the malloc function call and then bind the
//      public block to a private block using the function vsip_blockbind_f.
//      It will then call vsip_blockfind_f to obtain the pointer to the block.
//      After obtaining the pointer, it will compare values in this array to
//      verify that we have obtained the correct pointer. Also a set of
//      tests will create private blocks using the vsip_blockcreate_f and
//      vsip_cblockcreate_f functions and test whether the value returned
//      by vsip_blockfind_f is null.
//
//      Check some of the parameters of this function
//        1) Check the public/private bit (should be set to public (1) if a
//           non-null pointer is returned)
//        2) Check the pointer to the array that was returned from the function
//           and verify that the data is the same as initially used to fill
//           the array.
//
// VSIP functions called: vsip_blockbind_f, vsip_blockfind_f,
//                        vsip_blockcreate_f, vsip_blockdestroy_f,
//                        vsip_cblockcreate_f
//                        
//
// Revision History:
// May    6, 1997  Added copyright notice.             
// May    6, 1997  Executed tests 1-2 without error and block sizes up to 5000
// April 22, 1999  Adapted to c*_f

*/

#include<stdio.h>
#include<stdlib.h>  /* For malloc functionality */
#include "vsiptest_block.h"

int main()
{

  size_t                block_size, i;
  vsip_scalar_f        *array1, *array2, *array3;
  vsip_memory_hint      hint = 0;
  vsip_cblock_f        *private_block, *bind_block_s, *bind_block_i;
  int                   ret = 0;
  vsip_scalar_f        *find_block_1, *find_block_2;
  int                   j;

  vsip_init ((void *)0);
  /* Check various public block sizes from 2 to block_length_max */
  for (j=0; j<elements(VSIPTEST_CBLOCKFIND_F_BLOCK_SIZE); j++)
  {
    block_size = VSIPTEST_CBLOCKFIND_F_BLOCK_SIZE[j];
    array1 = (vsip_scalar_f*)malloc(block_size*sizeof(vsip_scalar_f));
    array2 = (vsip_scalar_f*)malloc(block_size*sizeof(vsip_scalar_f));
    array3 = (vsip_scalar_f*)malloc(block_size*sizeof(vsip_scalar_f)*2);

    if (ret)
    {
      break;
    }

    /* Fill the block with data values equal to its index value */
    for(i=0; i<block_size*2; i++) 
    {
      if (i<block_size)
      {
        *(array1 + i) = (vsip_scalar_f)i;
        *(array2 + i) = (vsip_scalar_f)i*2;
      }
      *(array3 + i) = (vsip_scalar_f)i*4;
    }

    /********************* VSIP_BLOCKBIND_D  ************************
      Bind to the public block with at least block_size real elements. */
    bind_block_s = vsip_cblockbind_f(array1, array2, block_size, hint);
    bind_block_i = vsip_cblockbind_f(array3, NULL, block_size, hint);

    /********************* VSIP_BLOCKFIND_D *************************
      Find the public block pointed to by bind_block */
    
    vsip_cblockfind_f (bind_block_s, &find_block_1, &find_block_2);

    if(find_block_1 == (vsip_scalar_f*)NULL || 
       find_block_2 == (vsip_scalar_f*)NULL   )
    {
      /* Report error */
      vtprintf("Error in vsip_cblockfind_f(); (1) \n"
               "Pointers to split block were not returned.\n");
      ret |= 1<<1;
      break;
    }
    else 
    {
      /* Check the data array's values for matching with public_block */
      for(i=0; i<block_size; i++) 
      {
        if(vtfcomp (*(array1 + i), *(find_block_1 + i)) ||
           vtfcomp (*(array2 + i), *(find_block_2 + i)) ) 
        {
          /* Report error */
          vtprintf("Error in vsip_cblockfind_f(); (2) \n"
                   "Array values from found split blocks do not match"
                   "original values\n");
          ret |= 1<<2;
          break;
        }
      }

      vsip_cblockfind_f (bind_block_i, &find_block_1, &find_block_2);

      if (find_block_2)
      {
        vtprintf ("Error in vsip_cblockfind_f(); (3)\n"
                  "Second pointer returned from interleaved block was "
                  "not NULL\n");
        ret |= 1<<3;
      }

      if (!find_block_1)
      {
        vtprintf ("Error in vsip_cblockfind_f(); (4)\n"
                  "Pointer to interleaved block was not returned\n");
        ret |= 1<<4;
      }

      else
      {
        for (i=0; i<block_size*2; i++)
        {
          if (vtfcomp (*(array3 + i), *(find_block_1 + i)) )
          {
            vtprintf ("Error in vsip_cblockbind_f(); (5)\n"
                      "Array values in interleaved block do not match"
                      " original values\n");
            ret |= 1<<5;
            break;
          }
        }
      }
    }

    free(array1);
    free(array2);
    free(array3);

    /********************* VSIP_BLOCKCREATE_D ***********************
      Create a private real-valued data block */
    private_block = vsip_cblockcreate_f(block_size, hint);

    /********************* VSIP_BLOCKFIND_D *************************
      Find if a call to block_find will return a public block in
      this case */
     vsip_cblockfind_f(private_block, &find_block_1, &find_block_2);

    if(find_block_1 != NULL) 
    {
      /* Report error */
      vtprintf("Error in vsip_blockfind_f(); (6) \n"
               "Pointer to VSIP block was returned from find_block \n");
      ret |= 1<<6;
      break;
    }

    /******************** VSIP_BLOCKDESTROY_D ***********************
      Destroy the block after checking all the attributes */
    vsip_cblockdestroy_f(private_block);

  }
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
