/* vsiptest_cblockrebind_f.c Version 0.000 April 22, 1999
//
// This file contains the vsiptest_cblockrebind_f functionality.
//
// Functional Description:
//   vsip_cblockrebind_f "rebinds" an existing released complex block object 
//   to a new user data array.  this array must be at least as big as the 
//   original.  an attempt to rebind a block in admitted state is an error, 
//   and should return null.  otherwise, a pointer to the old array should 
//   be returned.
//
// Copyright April 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description:
//    for split and interleaved
//      create data arrays A and B
//      bind a block to array A
//      admit block
//      attempt to rebind to B, verify error
//      release block
//      attempt to rebind to NULL, verify error
//      attempt to rebind to B, verify we returned a pointer to A
//    test view integrity switching from NULL bound blocks to split/interleave
//    test view integrity switching between split and interleaved
//      
// Revision History:
// March 26, 1999  Created 
// April 23, 1999  Converted to c*_f
*/


#include <stdio.h>
#include <stdlib.h>
#include "vsiptest_block.h"

int main()
{
  size_t                block_size = VSIPTEST_BLOCKREBIND_BLOCK_SIZE;
  vsip_memory_hint      hint = 0;
  vsip_cvview_f        *scview;
  vsip_cvview_f        *icview;
  vsip_vview_f         *siview; 
  vsip_vview_f         *srview; 
  vsip_vview_f         *iiview; 
  vsip_vview_f         *irview; 
  vsip_cblock_f        *sblock;
  vsip_cblock_f        *iblock;
  
  int i;
  int ret = 0;
  
  vsip_scalar_f        *array1, *array2, *array3, *array4, *array5, *array6;
  vsip_scalar_f        *prv_data1;
  vsip_scalar_f        *prv_data2;
  vsip_scalar_f        *prv_data3;
  vsip_scalar_f        *prv_data4;

  vsip_scalar_f         srval;
  vsip_scalar_f         sival;
  vsip_scalar_f         irval;
  vsip_scalar_f         iival;
 

  array1 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f) * 2);
  array4 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));
  array5 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));
  array6 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f) * 2);

  vsip_init ((void *)0);
 
  for (i=0; i<block_size; i++)
  {
    array1[i] = (vsip_scalar_f)    i;
    array2[i] = (vsip_scalar_f)2 * i;
    array4[i] = (vsip_scalar_f)4 * i;
    array5[i] = (vsip_scalar_f)5 * i;
  }
  for (i=0; i<block_size*2; i++)
  {
    array3[i] = (vsip_scalar_f)3 * i;
    array6[i] = (vsip_scalar_f)6 * i;
  }

  sblock = vsip_cblockbind_f (array1, array2, block_size, hint);
  iblock = vsip_cblockbind_f (array3, NULL, block_size, hint);
  
  if (vsip_cblockadmit_f (iblock, 1))
  {
      vtprintf ("Error in vsip_blockrebind_f(); (1) \n"
                "Could not admit interleaved test block\n");
      ret |= 1<<1;
  }

  if (vsip_cblockadmit_f (sblock, 1))
  {
      vtprintf ("Error in vsipblockrebind_f(); (2)\n"
                "Could not admit split test block\n");
      ret |= 1<<2;
  }

  vsip_cblockrebind_f (sblock, array4, array5, &prv_data1, &prv_data2);
  if (prv_data1 != NULL)
  {
      vtprintf ("Error in vsip_cblockrebind_f(); (3) \n"
                "Rebind of admitted split block did not return NULL\n");
      ret |= 1<<3;
  }

  vsip_cblockrebind_f (iblock, array6, NULL, &prv_data1, &prv_data2);
  if (prv_data1 != NULL)
  {
      vtprintf ("Error in vsip_cblockrebind_f(); (4)\n"
                "Rebind of admitted interleaved block did not return NULL\n");
      ret |= 1<<4;
  }

  vsip_cblockrelease_f (sblock, 0, &prv_data1, &prv_data2);
  if (!prv_data1 && !prv_data2)
  {
      vtprintf ("Error in vsip_blockrebind_f(); (5) \n"
                "Could not release split test block\n");
      ret |= 1<<5;
  }

  vsip_cblockrelease_f (iblock, 0, &prv_data1, &prv_data2);
  if (!prv_data1 && !prv_data2)
  {
      vtprintf ("Error in vsip_blockrebind_f(); (6) \n"
                "Could not release interleaved test block\n");
      ret |= 1<<6;
  }
  
  /*
  vsip_cblockrebind_f (sblock, NULL, NULL, &prv_data1, &prv_data2);
  if (prv_data1 != NULL)
  {
      vtprintf ("Error in vsip_cblockrebind_f(); (7) \n"
               "Rebind of split block to NULLs did not return NULL\n");
      ret |= 1<<7;
  }

  vsip_cblockrebind_f (iblock, NULL, NULL, &prv_data1, &prv_data2);
  if (prv_data1 != NULL)
  {
      vtprintf ("Error in vsip_cblockrebind_f(); (8) \n"
               "Rebind of interleaved block to NULLs did not return NULL\n");
      ret |= 1<<8;
  }
  */


  vsip_cblockdestroy_f (iblock);
  vsip_cblockdestroy_f (sblock);

  iblock = vsip_cblockbind_f (array3, NULL,   block_size, hint);
  sblock = vsip_cblockbind_f (array1, array2, block_size, hint);

  vsip_cblockrebind_f (iblock, array4, NULL, &prv_data1, &prv_data2);
  if (prv_data1 != array3)
  {
      vtprintf ("Error in vsip_cblockrebind_f(); (9) \n"
                "Rebind of interleaved block did not return old "
                "data pointer\n");
      ret |= 1<<9;
  }

  vsip_cblockrebind_f (sblock, array4, array5, &prv_data1, &prv_data2);
  if (prv_data1 != array1)
  {
      vtprintf ("Error in vsip_cblockrebind_f(); (10) \n"
                "Rebind of split block did not return old "
                "data pointer for real part\n");
      ret |= 1<<10;
  }
  if (prv_data2 != array2)
  {
      vtprintf ("Error in vsip_cblockrebind_f(); (11) \n"
                "Rebind of split block did not return old "
                "data pointer for imaginary part\n");
      ret |= 1<<11;
  }

  vsip_cblockdestroy_f (iblock);
  vsip_cblockdestroy_f (sblock);

  /* 
    Now test switching NULL->interleaved, NULL->split, interleaved->split
    and split->interleaved
  */

  sblock = vsip_cblockbind_f (array1, array2, block_size, hint);
  iblock = vsip_cblockbind_f (array3, NULL,   block_size, hint);

  scview = vsip_cvbind_f (sblock, 0, 1, block_size);
  icview = vsip_cvbind_f (iblock, 0, 1, block_size);

  siview = vsip_vimagview_f (scview);
  srview = vsip_vrealview_f (scview);

  iiview = vsip_vimagview_f (icview);
  irview = vsip_vrealview_f (icview);

  vsip_cblockrebind_f (sblock, array6, NULL,   &prv_data1, &prv_data2);
  vsip_cblockrebind_f (iblock, array4, array5, &prv_data3, &prv_data4);

  vsip_cblockadmit_f (sblock, 1);
  vsip_cblockadmit_f (iblock, 1);

  if (prv_data1 != array1 || prv_data2 != array2)
  {
    vtprintf ("Error in vsip_cblockrebind_f(); (12)\n"
              "Rebind of split block to interleaved did not return original "
              "data pointers\n");
    ret |= 1<<12;
  }

  if (prv_data3 != array3 || prv_data4 != NULL)
  {
    vtprintf ("Error in vsip_cblockrebind_f(); (13)\n"
              "Rebind of interleaved block to split did not return "
              "original data pointers\n");
    ret |= 1<<13;
  }

  for (i=0; i<block_size; i++)
  {
    sival = vsip_vget_f (siview, i);
    srval = vsip_vget_f (srview, i);

    if (vtfcomp (srval, (i * 2) * 6) || vtfcomp (sival, (i * 2 + 1) * 6) )
    {
      vtprintf ("Error in vsip_cblockrebind_f (14)\n"
                "Split block rebound to interleaved did not maintain view "
                "integrity\n");
      ret |= 1<<14;
      break;
    }
  }

  for (i=0; i<block_size; i++)
  {
    iival = vsip_vget_f (iiview, i);
    irval = vsip_vget_f (irview, i);

    if (vtfcomp (iival , i * 5) || vtfcomp (irval, i * 4))
    {
      vtprintf ("Error in vsip_cblockrebind_f (15)\n"
                "Interleaved block rebound to split did not maintain view "
                "integrity\n");
      ret |= 1<<15;
      break;
    }
  }

  vsip_vdestroy_f (siview);
  vsip_vdestroy_f (srview);
  vsip_vdestroy_f (iiview);
  vsip_vdestroy_f (irview);
  vsip_cvdestroy_f (icview);
  vsip_cvdestroy_f (scview);
  vsip_cblockdestroy_f (iblock);
  vsip_cblockdestroy_f (sblock);

  /* these tests may be bogus */

  sblock = vsip_cblockbind_f (NULL, NULL, block_size, hint);
  iblock = vsip_cblockbind_f (NULL, NULL, block_size, hint);

  scview = vsip_cvbind_f (sblock, 0, 1, block_size);
  icview = vsip_cvbind_f (iblock, 0, 1, block_size);

  siview = vsip_vimagview_f (scview);
  srview = vsip_vrealview_f (scview);

  iiview = vsip_vimagview_f (icview);
  irview = vsip_vrealview_f (icview);

  vsip_cblockrebind_f (sblock, array4, array5, &prv_data3, &prv_data4);
  vsip_cblockrebind_f (iblock, array6, NULL,   &prv_data1, &prv_data2);

  vsip_cblockadmit_f (sblock, 1);
  vsip_cblockadmit_f (iblock, 1);

  if (prv_data1 != NULL || prv_data2 != NULL)
  {
    vtprintf ("Error in vsip_cblockrebind_f(); (16)\n"
              "Rebind of NULL bound block to split did not return original "
              "data pointers\n");
    ret |= 1<<16;
  }

  if (prv_data3 != NULL || prv_data2 != NULL)
  {
    vtprintf ("Error in vsip_cblockrebind_f(); (17)\n"
              "Rebind of NULL bound block to interleaved did not return "
              "original data pointers\n");
    ret |= 1<<17;
  }

  for (i=0; i<block_size; i++)
  {
    sival = vsip_vget_f (siview, i);
    srval = vsip_vget_f (srview, i);

    if (vtfcomp (sival , i * 5) || vtfcomp (srval , i * 4))
    {
      vtprintf ("Error in vsip_cblockrebind_f (18)\n"
                "NULL bound block rebound to split did not maintain view "
                "integrity.\n");
      ret |= 1<<18;
      break;
    }
  }

  for (i=0; i<block_size; i++)
  {
    iival = vsip_vget_f (iiview, i);
    irval = vsip_vget_f (irview, i);

    if (vtfcomp (irval, (i * 2) * 6) || vtfcomp (iival, (i * 2 + 1) * 6))
    {
      vtprintf ("Error in vsip_cblockrebind_f (19)\n"
                "NULL bound block rebound to interleaved did not maintain view "
                "integrity\n");
      ret |= 1<<19;
      break;
    }
  }

  vsip_vdestroy_f (siview);
  vsip_vdestroy_f (srview);
  vsip_vdestroy_f (iiview);
  vsip_vdestroy_f (irview);
  vsip_cvdestroy_f (icview);
  vsip_cvdestroy_f (scview);
  vsip_cblockdestroy_f (iblock);
  vsip_cblockdestroy_f (sblock);
  
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}

