/* vsiptest_realscalar.c Version 0.000 September 3, 1999
// 
// This file contains the source code included for testing elementwise
// functions which take a vector and a scalar as arguments
// 
// Copyright September 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) with unit strides, test each of the 8 possible types of
//         real blocks:
//         VR   a) VSIP real block
//         VRC  b) Derived from real part of VSIP complex block
//         VRI  c) Derived from imaginary part of VSIP complex block
//         UR   d) User real block
//         URCS e) Derived from real part of user split complex block
//         UICS f) Derived from imag part of user split complex block
//         URCI g) Derived from real part of user inter. complex block
//         UICI h) Derived from imag part of user inter. complex block
//          S ~ VR   = VRC
//          S ~ VRI  = UR
//          S ~ URCS = UICI
//          S ~ UICS = URCI
//      2) For various stride combinations,
//          S ~ UR = VR
//      3) Test in-place operation:
//          S ~ VR = VR
//          S ~ UR = UR
//
// Revision History:
// September 2, 1999   Created
// March 20,    2002   [dpc] fixed some erroneous alldestroys
*/

#include <math.h>

/* Input #defines:
        VSIPTEST_ELWISE_REAL_UNARY     
        INPUT_FILENAME          results vector file
        FUNCTION_NAME           string desc. of function to be tested
        TEST_FUNCTION           pointer to vsip_ function to be tested
        VSIPTEST_EPSILON_MAX    max difference for this test
        VSIPTEST_THOROUGH       defined if thorough tests are to be run
        VSIPTEST_EPSILON_MAX    Maximum absolute value error for 
                                results comparison

        VSIPTEST_ELWISE_DO_EPSILON_TEST if defined, do the diff. test
        VSIPTEST_ELWISE_DO_SIGN_TEST    if defined, do the sign test
*/


char filename[] = INPUT_FILENAME;

int vttest_function (char *filename, 
                     void (*function) (vsip_scalar_f,
                                       const vsip_vview_f *, 
                                       const vsip_vview_f *),
                     vsip_vview_f *view1, vsip_vview_f *view2, 
                     int length, char *function_name, int first_error)
{
  FILE *fp = fopen (filename, "r");
  int ret = 0;
  int read = 0;
  int val = 0;
  int i;
  vsip_scalar_f rval, val1, val2;
  vsip_scalar_f scalar;

  if (!fp)
  {
    vtprintf ("Error in %s (%i)\n"
              "Could not open %s for reading\n", function_name,
              first_error+0, filename);
    ret |= 1<<(first_error+0);
    return (ret); 
  }

  val = vtgetlength (fp);

  if (val < length)
  {
    vtprintf ("Error in %s (%i)\n"
              "length (%i) in data file(%s)\n"
              "is shorter than views for testing(%i)\n", function_name,
              first_error+0, val, filename, length);
    ret |= 1<<(first_error+0);
    fclose (fp);
    return (ret); 
  }

  /* get the scalar */

  read = fscanf (fp, "%f\n", &scalar);
  if (!read)
  {
    vtprintf ("Error in %s (%i)\n",
              "Could not read scalar value for function\n", 
              function_name, first_error);
    ret |= 1<<first_error;
  }

  /* load the operand vectors */
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f %f%*[^\n]\n", &val1, &val2);
    if (read<2)
    {
      vtprintf ("Error in %s (%i)\n"
                "Read %i elements in row %i of data file %s\n"
                "expecting 2\n", function_name, first_error+0, filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    vsip_vput_f (view1, i, val1);
  }
  fclose (fp);
  
  /* do the function */

  function (scalar, view1, view2);

  /* check the results */

  fp = fopen (filename, "r");
  vtgetlength (fp);
  fscanf (fp, "%f\n", &scalar);
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f %f\n", &val1, &val2);
    if (read<2)
    {
      vtprintf ("Error in %s (%i)\n"
                "Could not read result value for row %i of "
                "data file %s\n", function_name, first_error+0,
                filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    rval = vsip_vget_f (view2, i);
    update_error_element (fabs(rval-val2));

#ifdef VSIPTEST_ELWISE_DO_EPSILON_TEST
    if (fabs (rval - val2) > VSIPTEST_EPSILON_MAX)
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed closeness test - expected %f "
                "got %f\ninput=(%f)\n", function_name, first_error+1, 
                i, val2, rval, vsip_vget_f (view1, i));
      ret |= 1<<(first_error+1);
    }
#endif

#ifdef VSIPTEST_ELWISE_DO_SIGN_TEST
    if ( (val2 <  0 && rval >= 0) ||
         (val2 >  0 && rval <= 0) ||
         (val2 != 0 && rval == 0) )
    {
      vtprintf ("Error in %s (%i)\n"
                "Element %i failed sign test - expected %f got %f\n"
                "input=(%f)\n",
                function_name, first_error+2, i, val2, rval,
                vsip_vget_f (view1, i));
      ret |= 1<<(first_error+2);
    }
#endif

    if (ret)
    {
      fclose (fp);
      return (ret); 
    }
  }
  update_error_total();

  fclose (fp);
  return (ret); 
}

int main(int argc, char *argv[])
{
  int length;
  int ret = 0;
  vsip_vview_f *VR, *VR2, *VRC, *VRI, *UR, *UR2, *URCS, 
               *URCI, *UICS, *UICI;
  vsip_cvview_f *VC;
  vsip_block_f  *vblock, *ublock;
  vsip_cblock_f *csplitb, *cinterb;
  vsip_cvview_f *csplitv, *cinterv;
  vsip_scalar_f *array1, *array2, *array3, *array4;
  int i;
  vsip_stride lstride, rstride;
  vsip_offset offset;
  vsip_length templen;

  vsip_init ((void *)0);

  length = vtelwise_get_length (filename);
  if (!length)
  {
    vtprintf ("Error in %s (1)\n"
              "Could not get length from datafile %s\n",
              FUNCTION_NAME, filename);
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  VR = vsip_vcreate_f (length, 0);
  VC = vsip_cvcreate_f (length, 0);
  VRC = vsip_vrealview_f (VC);
  VRI = vsip_vimagview_f (VC);

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array4 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*length);

  vblock = vsip_blockcreate_f (length, 0);
  ublock = vsip_blockbind_f (array1, length, 0);
	vsip_blockadmit_f (ublock, 0);

  UR = vsip_vbind_f (ublock, 0, 1, length);
  csplitb = vsip_cblockbind_f (array2, array3, length, 0);
  cinterb = vsip_cblockbind_f (array4, NULL, length,0);
	vsip_cblockadmit_f (csplitb, 0);
	vsip_cblockadmit_f (cinterb, 1);


  csplitv = vsip_cvbind_f (csplitb, 0, 1, length);
  cinterv = vsip_cvbind_f (cinterb, 0, 1, length);
  URCS = vsip_vrealview_f (csplitv);
  URCI = vsip_vrealview_f (cinterv);
  UICS = vsip_vimagview_f (csplitv);
  UICI = vsip_vimagview_f (cinterv);

  error_init (argv[1]);

  if (!VR || !VRI || !VRC || !URCS || !URCI || !UICS || !UICI)
  {
    vtprintf ("Error in %s (2)\n"
              "Could not allocate all test vectors\n", FUNCTION_NAME);
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (ret |= vttest_function (filename, TEST_FUNCTION, VR, VRC, 
                              length, FUNCTION_NAME, 3))
  {
    vtprintf ("VSIPL created block into block derived from real portion "
              "of VSIPL created\n"
              "complex block testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("VSIPL created block into block derived from real portion "
                "of VSIPL created\ncomplex block test");

#ifndef VSIPTEST_THOROUGH
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
#endif

  if (ret |= vttest_function (filename, TEST_FUNCTION, VRI, UR, 
                              length, FUNCTION_NAME, 6))
  {
    vtprintf ("Block derived from imaginary portion of VSIP created "
              "complex block\n"
              "into user created block testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("Block derived from imaginary portion of VSIP created "
                     "complex block\ninto user created block test");

  if (ret |= vttest_function (filename, TEST_FUNCTION, URCS, UICI, 
                              length, FUNCTION_NAME, 9))
  {
    vtprintf ("Block derived from real part of user created split "
              "complex block into\n"
              "block derived from imaginary part of user created "
              "interleaved complex block\n"
              "testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("Block derived from real part of user created split "
                "complex block into\n"
                "block derived from imaginary part of user created "
                "interleaved complex block test");

  if (ret |= vttest_function (filename, TEST_FUNCTION, UICS, URCI, 
                              length, FUNCTION_NAME, 12))
  {
    vtprintf ("Block derived from imaginary part of user created split "
              "complex block\n"
              "into block derived from real part of user created "
              "interleaved block\n"
              "testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("lock derived from imaginary part of user created split "
              "complex block\n"
              "into block derived from real part of user created "
              "interleaved block test");


  /* Stride tests */
  for (i=0; i<elements(VSIPTEST_ELWISE_LEFT_STRIDES); i++)
  {
    lstride = VSIPTEST_ELWISE_LEFT_STRIDES[i];
    rstride = VSIPTEST_ELWISE_RIGHT_STRIDES[i];
    if (rstride == 1)
    {
      offset = 0;
    }
    else if (rstride == -1)
    {
      offset = length-1;
    }
    else
    {
      vtprintf ("Illegal stride for right stride of stride test in\n"
                "Must be 1 or -1\n");
      report (-1); 
    vsip_finalize ((void *)0);
      return (-1 != 0);
    }
    templen = length / lstride;

    VR2 = vsip_vbind_f (vblock, 0, lstride, templen);
    UR2 = vsip_vbind_f (ublock, offset, rstride, templen);

    if (ret |= vttest_function (filename, TEST_FUNCTION, VR2, UR2,
                                templen, FUNCTION_NAME, 15) )
    {
      vtprintf ("Stride test (%i)->%i produced errors\n", lstride,
                rstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("Stride test (%i)->%i", lstride);

    vsip_vdestroy_f (VR2);
    vsip_vdestroy_f (UR2);
  }

  /* In place test */

  if (ret |= vttest_function (filename, TEST_FUNCTION, VR, UR, 
                              length, FUNCTION_NAME, 18) )
  {
    vtprintf ("In place test produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("In place test");
  error_finish();
  report (ret); 

  vsip_blockdestroy_f (vblock);
  vsip_vdestroy_f (URCS);
  vsip_vdestroy_f (URCI);
  vsip_vdestroy_f (UICS);
  vsip_vdestroy_f (UICI);
  vsip_valldestroy_f (UR);
  vsip_valldestroy_f (VR);
  vsip_vdestroy_f (VRC);
  vsip_vdestroy_f (VRI);
  vsip_cvalldestroy_f (csplitv);
  vsip_cvalldestroy_f (cinterv);
  vsip_cvalldestroy_f (VC);

  free (array1);
  free (array2);
  free (array3);
  free (array4);

  vsip_finalize ((void *)0);
  return (ret != 0);
}
