/* vsiptest_realtoscalar2.c Version 0.000 September 3, 1999
// 
// This file contains the source code included for testing real unary
// element wise functions that return two scalars
// 
// Copyright September 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) with unit strides, test each of the 8 possible types of
//         real blocks:
//         VR   a) VSIP real block
//         VRC  b) Derived from real part of VSIP complex block
//         VRI  c) Derived from imaginary part of VSIP complex block
//         UR   d) User real block
//         URCS e) Derived from real part of user split complex block
//         UICS f) Derived from imag part of user split complex block
//         URCI g) Derived from real part of user inter. complex block
//         UICI h) Derived from imag part of user inter. complex block
//
// Revision History:
// September 3, 1999   Created
// March 20, 2002 [dpc] fixed some erroneous alldestroys
*/

#include <math.h>

/* Input #defines:
        VSIPTEST_ELWISE_REAL_UNARY     
        INPUT_FILENAME          results vector file
        FUNCTION_NAME           string desc. of function to be tested
        TEST_FUNCTION           pointer to vsip_ function to be tested
        VSIPTEST_EPSILON_MAX    max difference for this test
        VSIPTEST_THOROUGH       defined if thorough tests are to be run
        VSIPTEST_EPSILON_MAX    Maximum absolute value error for 
                                results comparison

        VSIPTEST_ELWISE_DO_EPSILON_TEST if defined, do the diff. test
        VSIPTEST_ELWISE_DO_SIGN_TEST    if defined, do the sign test
*/


char filename[] = INPUT_FILENAME;

int vttest_function (char *filename, 
                     vsip_scalar_f (*function) (const vsip_vview_f *,
                                                vsip_scalar_vi *),
                     vsip_vview_f *view1, int length, 
                     char *function_name, int first_error)
{
  int tmp;
  FILE *fp = fopen (filename, "r");
  int ret = 0;
  int read = 0;
  int val = 0;
  int i;
  vsip_scalar_f rval, val1;
  vsip_scalar_vi rval2, val2;

  if (!fp)
  {
    vtprintf ("Error in %s (%i)\n"
              "Could not open %s for reading\n", function_name,
              first_error+0, filename);
    ret |= 1<<(first_error+0);
    return (ret); 
  }

  val = vtgetlength (fp);

  if (val < length)
  {
    vtprintf ("Error in %s (%i)\n"
              "length (%i) in data file(%s)\n"
              "is shorter than views for testing(%i)\n", function_name,
              first_error+0, val, filename, length);
    ret |= 1<<(first_error+0);
    fclose (fp);
    return (ret); 
  }

  /* get the result scalar */
  read = fscanf (fp, "%f\n", &rval);
  if (!read)
  {
    vtprintf ("Error in %s (%i)\n"
              "Could not read expected result in data file (%s)\n", 
              function_name, first_error, filename);
    ret |= 1<<first_error;
    fclose (fp);
    return (ret); 
  }

  /* read = fscanf (fp, "%d\n", (int *)&rval2);*/
  read = fscanf (fp, "%d\n", &tmp);
  if (!read)
  {
    vtprintf ("Error in %s (%i)\n"
              "Could not read second expected result in data file "
              "(%s)\n", function_name, first_error, filename);
    ret |= 1<<first_error;
    fclose (fp);
    return (ret); 
  }
	rval2 = (vsip_index)tmp;

  /* load the operand vectors */
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f*[^\n]\n", &val1);
    if (read<1)
    {
      vtprintf ("Error in %s (%i)\n"
                "Read %i elements in row %i of data file %s\n"
                "expecting 1\n", function_name, first_error+0, filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    vsip_vput_f (view1, i, val1);
  }
  fclose (fp);
  
  /* do the function */

  val1 = function (view1, &val2);
  update_error_element (fabs(rval-val1));

  /* check the results */

#ifdef VSIPTEST_ELWISE_DO_EPSILON_TEST
  if (fabs (rval - val1) > VSIPTEST_EPSILON_MAX ||
      rval2 != val2 )
  {
    vtprintf ("Error in %s (%i)\n"
              "Function failed closeness test - expected %f,%i "
              "got %f,%i\n", function_name, first_error+1, 
               rval, rval2, val1, val2);
    ret |= 1<<(first_error+1);
  }
#endif

#ifdef VSIPTEST_ELWISE_DO_SIGN_TEST
  if ( (val1 <  0 && rval >= 0) ||
       (val1 >  0 && rval <= 0) ||
       (val1 != 0 && rval == 0) )
  {
    vtprintf ("Error in %s (%i)\n"
              "Function failed sign test - expected %f,%i got %f,%i\n",
              function_name, first_error+2, rval, rval2, val1, val2);
    ret |= 1<<(first_error+2);
  }
#endif

  update_error_total();

  return (ret); 
}

int main(int argc, char *argv[])
{
  int length;
  int lstride;
  int ret = 0;
  vsip_vview_f *VR, *VRC, *VRI, *UR, *URCS, 
               *URCI, *UICS, *UICI;
  vsip_cvview_f *VC;
  vsip_block_f  *ublock;
  vsip_cblock_f *csplitb, *cinterb;
  vsip_cvview_f *csplitv, *cinterv;
  vsip_scalar_f *array1, *array2, *array3, *array4;
  vsip_length templen;

  vsip_init ((void *)0);

  length = vtelwise_get_length (filename);
  if (!length)
  {
    vtprintf ("Error in %s (1)\n"
              "Could not get length from datafile %s\n",
              FUNCTION_NAME, filename);
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  VR = vsip_vcreate_f (length, 0);
  VC = vsip_cvcreate_f (length, 0);
  VRC = vsip_vrealview_f (VC);
  VRI = vsip_vimagview_f (VC);

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array4 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*length);

  ublock = vsip_blockbind_f (array1, length, 0);
  UR = vsip_vbind_f (ublock, 0, 1, length);
  csplitb = vsip_cblockbind_f (array2, array3, length, 0);
  cinterb = vsip_cblockbind_f (array4, NULL, length,0);
  csplitv = vsip_cvbind_f (csplitb, 0, 1, length);
  cinterv = vsip_cvbind_f (cinterb, 0, 1, length);
  URCS = vsip_vrealview_f (csplitv);
  URCI = vsip_vrealview_f (cinterv);
  UICS = vsip_vimagview_f (csplitv);
  UICI = vsip_vimagview_f (cinterv);

  error_init (argv[1]);

  if (!VR || !VRI || !VRC || !URCS || !URCI || !UICS || !UICI)
  {
    vtprintf ("Error in %s (2)\n"
              "Could not allocate all test vectors\n", FUNCTION_NAME);
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  templen = length;
  lstride = 1;

    if (ret |= vttest_function (filename, TEST_FUNCTION, VR,
                                templen, FUNCTION_NAME, 3) )
    {
      vtprintf ("Stride test (%i) produced errors for VSIPL created "
                "real block\n", lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    
    error_report_test ("VSIPL created real block stride test (%i)", lstride);

    if (ret |= vttest_function (filename, TEST_FUNCTION, VRC,
                                templen, FUNCTION_NAME, 6) )
    {
      vtprintf ("Stride test (%i) produced errors for block derived from "
                "real part of\nVSIPL created complex block\n", lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("Block derived from real part of VSIPL created complex block "
                       "stride test (%i)", lstride);

    if (ret |= vttest_function (filename, TEST_FUNCTION, VRI,
                                templen, FUNCTION_NAME, 3) )
    {
      vtprintf ("Stride test (%i) produced errors for block derived from "
                "imaginary part of\nVSIP created complex block\n", 
                lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("Block derived from imaginary part of VSIPL created complex block "
                       "stride test (%i)", lstride);

    if (ret |= vttest_function (filename, TEST_FUNCTION, UR,
                                templen, FUNCTION_NAME, 3) )
    {
      vtprintf ("Stride test (%i) produced errors for user created"
                "real block\n", lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("User created real block strde test (%i)", lstride);

    if (ret |= vttest_function (filename, TEST_FUNCTION, URCS,
                                templen, FUNCTION_NAME, 3) )
    {
      vtprintf ("Stride test (%i) produced errors for view derived from "
                "real part of user\ncreated split complex block\n", 
                lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("View derived from real part of user created split complex block" 
                       "stride test (%i)", lstride);

    if (ret |= vttest_function (filename, TEST_FUNCTION, URCI,
                                templen, FUNCTION_NAME, 3) )
    {
      vtprintf ("Stride test (%i) produced errors for view derived from "
                "real part of user\ncreated interleaved complex block\n", 
                lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("View derived from real part of user created interleaved block" 
                       "stride test (%i)", lstride);

    if (ret |= vttest_function (filename, TEST_FUNCTION, UICS,
                                templen, FUNCTION_NAME, 3) )
    {
      vtprintf ("Stride test (%i) produced errors for view derived from "
                "imaginary  part of user\ncreated split complex block\n", 
                lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("View derived from imaginary part of user created split block" 
                       "stride test (%i)", lstride);

    if (ret |= vttest_function (filename, TEST_FUNCTION, UICI,
                                templen, FUNCTION_NAME, 3) )
    {
      vtprintf ("Stride test (%i) produced errors for view derived from "
                "imaginary  part of user\ncreated interleaved complex "
                "block\n", lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    error_report_test ("View derived from imaginary part of user created interleaved block" 
                       "stride test (%i)", lstride);
  error_finish();
  report (ret); 

  vsip_valldestroy_f (VR);
  vsip_vdestroy_f (VRC);
  vsip_vdestroy_f (VRI);
  vsip_valldestroy_f (UR);
  vsip_vdestroy_f (URCS);
  vsip_vdestroy_f (URCI);
  vsip_vdestroy_f (UICS);
  vsip_vdestroy_f (UICI);
  vsip_cvalldestroy_f (VC);
  vsip_cvalldestroy_f (csplitv);
  vsip_cvalldestroy_f (cinterv);

  vsip_finalize ((void *)0);
  return (ret != 0);
}
