/* vsiptest_complextoreal.c Version 0.000 September 3, 1999,
// 
// This file contains the source code included for testing element wise
// functions which take complex vectors as input, and produce real
// real vectors as output
// 
// Copyright September 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) with unit stride, test each of the 3 
//         possible types of complex blocks
//         V    a) VSIP block
//         US   b) User created split complex block
//         UI   c) User created interleaved complex block
//
// Revision History:
// September 2, 1999   Created
*/

#include <math.h>
#include "vsiptest_elwise.h"

/* Input #defines:
        INPUT_FILENAME          results vector file
        FUNCTION_NAME           string desc. of function to be tested
        TEST_FUNCTION           pointer to vsip_ function to be tested
        VSIPTEST_EPSILON_MAX    max difference for this test
        VSIPTEST_THOROUGH       defined if thorough tests are to be run
*/


int vttest_function (char *filename, vsip_cvview_f *view1, int length,
                        int first_error)
{
  FILE *fp = fopen (filename, "r");
  int ret = 0;
  int read = 0;
  int val = 0;
  int i;
  vsip_scalar_f  val1, val2;
  vsip_scalar_f rrval, irval;
  vsip_cscalar_f rval;

  vsip_vview_f *v1, *v2;

  v1 = vsip_vcreate_f (length, 0);
  v2 = vsip_vcreate_f (length, 0);

  if (!v1 || !v2)
  {
    vtprintf ("Error in vsip_vcmplx_f (%i)\n"
              "Could not create test vectors\n");
    return (1<<first_error); 
  }

  if (!fp)
  {
    vtprintf ("Error in vsip_vcmplx_f (%i)\n"
              "Could not open %s for reading\n", first_error+0, filename);
    ret |= 1<<(first_error+0);
    return (ret); 
  }

  val = vtgetlength (fp);

  if (val < length)
  {
    vtprintf ("Error in vsip_vcmplx_f (%i)\n"
              "length (%i) in data file(%s)\n"
              "is shorter than views for testing(%i)\n", 
              first_error+0, val, filename, length);
    ret |= 1<<(first_error+0);
    fclose (fp);
    return (ret); 
  }

  /* load the operand vectors */
  for (i=0; i<length; i++)
  {
    read = fscanf (fp, "%f %f*[^\n]\n", 
                   &val1, &val2);
    if (read<2)
    {
      vtprintf ("Error in vsip_vcmplx_f (%i)\n"
                "Read %i elements in row %i of data file %s\n"
                "expecting 2\n", first_error+0, read, 
                i, filename);
      ret |= 1<<(first_error+0);
      fclose (fp);
      return (ret); 
    }
    vsip_vput_f (v1, i, val1);
    vsip_vput_f (v2, i, val2);
  }
  fclose (fp);
  
  /* do the function */

  vsip_vcmplx_f (v1, v2, view1);

  /* check the results */

  for (i=0; i<length; i++)
  {
    rval = vsip_cvget_f (view1, i);

    rrval = vsip_real_f (rval);
    irval = vsip_imag_f (rval);

    val1 = vsip_vget_f (v1, i);
    val2 = vsip_vget_f (v2, i);

    update_error_element (sqrt((rrval-val1)*(rrval-val1)+(irval-val2)*(irval-val2)));

    if ((fabs (rrval - val1) > VT_EM) ||
        (fabs (irval - val2) > VT_EM) )
    {
      vtprintf ("Error in vsip_vcmplx_f (%i)\n"
                "Element %i failed closeness test - \nexpected %f,%f "
                "got %f,%f\n",
                first_error+1, i, val1, val2 , rrval, irval);
      ret |= 1<<(first_error+1);
    }

    if ( (val1 <  0 && rrval >= 0) ||
         (val1 >  0 && rrval <= 0) ||
         (val1 != 0 && rrval == 0) ||
         (val2 <  0 && irval >= 0) ||
         (val2 >  0 && irval <= 0) ||
         (val2 != 0 && irval == 0) )
    {
      vtprintf ("Error in vsip_vcmplx_f (%i)\n"
                "Element %i failed sign test - \n"
                "expected %f,%f got %f,%f\n" ,
                first_error+2, i, val1, val2,
                rrval, irval);
      ret |= 1<<(first_error+2);
    }

    if (ret)
    {
      return (ret); 
    }
  }
  update_error_total();

  return (ret); 
}

char filename[] = "data/vcmplx_f";

int main(int argc, char *argv[])
{
  int length;
  int ret = 0;
  vsip_cvview_f *V, *US, *UI;
  vsip_vview_f *VR;
  vsip_cblock_f *csplitb, *cinterb;
  vsip_scalar_f *array1, *array2, *array3;

  vsip_init ((void *)0);

  length = vtelwise_get_length (filename);
  if (!length)
  {
    vtprintf ("Error in vsip_vcmplx_f (1)\n"
              "Could not get length from datafile %s\n",
              filename);
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  V = vsip_cvcreate_f (length, 0);
  VR = vsip_vcreate_f (length, 0);

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*length);

  csplitb = vsip_cblockbind_f (array1, array2, length, 0);
  cinterb = vsip_cblockbind_f (array3, NULL, length,0);
	vsip_cblockadmit_f (csplitb, 0);
	vsip_cblockadmit_f (cinterb, 0);


  US = vsip_cvbind_f (csplitb, 0, 1, length);
  UI = vsip_cvbind_f (cinterb, 0, 1, length);

  error_init (argv[1]);

  if (!V)
  {
    vtprintf ("Error in %s (2)\n"
              "Could not allocate all test vectors\n", "vsip_vcmplx_f");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (ret |= vttest_function (filename, V, length,3))
  {
    vtprintf ("VSIPL created complex block testing produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("VSIPL created complex block test");

  if (ret |= vttest_function (filename, US, length,6))
  {
    vtprintf ("User created split complex block testing "
              "produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("User created split complex block test");

  if (ret |= vttest_function (filename, UI, length,9))
  {
    vtprintf ("User created interleaved complex block testing "
              "produced errors\n");
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  error_report_test ("User created interleaved complex block test");

  error_finish();
  vsip_cvalldestroy_f (UI);
  vsip_cvalldestroy_f (US);
  vsip_valldestroy_f (VR);

  free (array1);
  free (array2);
  free (array3);

  report (ret); 
    vsip_finalize ((void *)0);
  return (ret != 0);
}
