/* vsiptest_vramp_f.c Version 0.000 September 3, 1999
// 
// This file contains the source code included for testing 
// vsip_vramp_f
// 
// Copyright September 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) with unit strides, test each of the 8 possible types of
//         real blocks:
//         VR   a) VSIP real block
//         VRC  b) Derived from real part of VSIP complex block
//         VRI  c) Derived from imaginary part of VSIP complex block
//         UR   d) User real block
//         URCS e) Derived from real part of user split complex block
//         UICS f) Derived from imag part of user split complex block
//         URCI g) Derived from real part of user inter. complex block
//         UICI h) Derived from imag part of user inter. complex block
//
// Revision History:
// September 3, 1999   Created
// March 20, 2002 [dpc] fixed some erroneous alldestroys
*/

#include <math.h>
#include "vsip.h"
#include "vsiptest_elwise.h"

/* Input #defines:
        VSIPTEST_ELWISE_REAL_UNARY     
        INPUT_FILENAME          results vector file
        FUNCTION_NAME           string desc. of function to be tested
        TEST_FUNCTION           pointer to vsip_ function to be tested
        VSIPTEST_EPSILON_MAX    max difference for this test
        VSIPTEST_THOROUGH       defined if thorough tests are to be run
        VSIPTEST_EPSILON_MAX    Maximum absolute value error for 
                                results comparison

        VSIPTEST_ELWISE_DO_EPSILON_TEST if defined, do the diff. test
        VSIPTEST_ELWISE_DO_SIGN_TEST    if defined, do the sign test
*/


int vttest_function (vsip_vview_f *view, int length, int errbase)
{
  int i,j;
  vsip_scalar_f val1, val2, dval;
  for (i = 0; i<elements(VSIPTEST_VRAMP_F_MINVALUES); i++)
  {
    val1 = VSIPTEST_VRAMP_F_MINVALUES[i];
    val2 = VSIPTEST_VRAMP_F_STEPS[i];

    vsip_vramp_f (val1, val2, view);

    for (j=0; j<length; j++)
    {
      dval = val1 + (double)j * val2;
      if (vtfcomp ( vsip_vget_f(view, j) , dval))
      {
        vtprintf ("Error in vsip_vfill_f (%i)\n"
                  "Element %i(%f) did not match expected value %f\n",
                  errbase, j, vsip_vget_f (view, j), dval);
        return (1<<errbase); 
      }
    }
  }
  return (0); 
}

int main()
{
  int length;
  int lstride;
  int ret = 0;
  vsip_vview_f *VR, *VRC, *VRI, *UR, *URCS, 
               *URCI, *UICS, *UICI;
  vsip_cvview_f *VC;
  vsip_block_f  *ublock;
  vsip_cblock_f *csplitb, *cinterb;
  vsip_cvview_f *csplitv, *cinterv;
  vsip_scalar_f *array1, *array2, *array3, *array4;
  vsip_length templen;

  length = VSIPTEST_VFILL_F_LENGTH;

  VR = vsip_vcreate_f (length, 0);
  VC = vsip_cvcreate_f (length, 0);
  VRC = vsip_vrealview_f (VC);
  VRI = vsip_vimagview_f (VC);

  vsip_init ((void *)0);

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*length);
  array4 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*length);

  ublock = vsip_blockbind_f (array1, length, 0);
	vsip_blockadmit_f (ublock, 0);

  UR = vsip_vbind_f (ublock, 0, 1, length);
  csplitb = vsip_cblockbind_f (array2, array3, length, 0);
  cinterb = vsip_cblockbind_f (array4, NULL, length,0);
  vsip_cblockadmit_f (csplitb, 0);
  vsip_cblockadmit_f (cinterb, 0);

  csplitv = vsip_cvbind_f (csplitb, 0, 1, length);
  cinterv = vsip_cvbind_f (cinterb, 0, 1, length);
  URCS = vsip_vrealview_f (csplitv);
  URCI = vsip_vrealview_f (cinterv);
  UICS = vsip_vimagview_f (csplitv);
  UICI = vsip_vimagview_f (cinterv);

  if (!VR || !VRI || !VRC || !URCS || !URCI || !UICS || !UICI)
  {
    vtprintf ("Error in %s (2)\n"
              "Could not allocate all test vectors\n", "vsip_vfill_f");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  templen = length;
  lstride = 1;

    if (ret |= vttest_function (VR, templen, 3))
    {
      vtprintf ("Stride test (%i) produced errors for VSIP created "
                "real block\n", lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    if (ret |= vttest_function (VRC, templen, 6))
    {
      vtprintf ("Stride test (%i) produced errors for block derived from "
                "real part of\nVSIP created complex block\n", lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    if (ret |= vttest_function (VRI, templen, 9))
    {
      vtprintf ("Stride test (%i) produced errors for block derived from "
                "imaginary part of\nVSIP created complex block\n", 
                lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    if (ret |= vttest_function (UR, templen, 12))
    {
      vtprintf ("Stride test (%i) produced errors for user created "
                "real block\n", lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    if (ret |= vttest_function (URCS, templen, 15))
    {
      vtprintf ("Stride test (%i) produced errors for view derived from "
                "real part of user\ncreated split complex block\n", 
                lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    if (ret |= vttest_function (URCI, templen, 18))
    {
      vtprintf ("Stride test (%i) produced errors for view derived from "
                "real part of user\ncreated interleaved complex block\n", 
                lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    if (ret |= vttest_function (UICS, templen, 21))
    {
      vtprintf ("Stride test (%i) produced errors for view derived from "
                "imaginary  part of user\ncreated split complex block\n", 
                lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    if (ret |= vttest_function (UICI, templen, 24))
    {
      vtprintf ("Stride test (%i) produced errors for view derived from "
                "imaginary  part of user\ncreated interleaved complex "
                "block\n", lstride);
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
  report (ret); 

  vsip_vdestroy_f (URCS);
  vsip_vdestroy_f (URCI);
  vsip_vdestroy_f (UICS);
  vsip_vdestroy_f (UICI);
  vsip_valldestroy_f (UR);
  vsip_valldestroy_f (VR);
  vsip_vdestroy_f (VRC);
  vsip_vdestroy_f (VRI);
  vsip_cvalldestroy_f (csplitv);
  vsip_cvalldestroy_f (cinterv);
  vsip_cvalldestroy_f (VC);

  free (array1);
  free (array2);
  free (array3);
  free (array4);


  vsip_finalize ((void *)0);
  return (ret != 0);
}
