/* vsiptest_cfirflt_f.c Version 0.000 November 8, 1999
// 
// This file contains the vsiptest_cfirflt_f test functionality.
// 
// Functional Description:
//   vsip_cfirflt_f does a complex FIR filter and decimates the output
//  
//
// Copyright November 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Revision History:
// Nov  8 1999 - Created
*/

#include"vsiptest_sigproc.h"
#include <stdio.h>

void vector_read (char *filename, vsip_cvview_f **input, 
                  vsip_cvview_f **output, vsip_cvview_f **kernel, 
                  vsip_symmetry *symm, vsip_length *N, vsip_length *D, 
                  vsip_obj_state *state, vsip_length *ntimes)
{
  FILE *fp;
  int read = 0;
  int i;
  float val1, val2, val3, val4, val5, val6;
  int n, d, nt, kernel_size, sym;
  int s;

  fp = fopen (filename, "r");
  if (!fp)
  {
    vtprintf ("Error in vsip_cfirflt_f (0)\n"
              "unable to open data file %s for reading\n", filename);
    exit(1);
  }

  read = fscanf (fp, "%i %i %i %i %i %i\n", &sym, &n, &d, &s, &nt, &kernel_size);
  *D = d;
  *ntimes = nt;
  *N = n;
  *symm = sym;
  *state = s;

  if (read != 6)
  {
    vtprintf ("Error in vsip_cfirflt_f (0)\n"
              "unable to read FIR parameters in %s\n", filename);
    fclose (fp);
    exit (1);
  }
  
  /* this is expected order in input file */
  *input = vsip_cvcreate_f (n, 0);
  *output = vsip_cvcreate_f (n, 0);
  *kernel = vsip_cvcreate_f (kernel_size, 0);

  if (!input || !output || !kernel)
  {
    vtprintf ("Error in vsip_cfirflt_f (0)\n"
              "unable to create views of size %i for %s\n", n, filename);
    fclose (fp);
    exit(1);
  }

  for (i=0; i<n; i++)
  {
    read = fscanf (fp, "%f,%f %f,%f %f,%f\n", &val1, &val2, &val3, &val4, &val5, &val6);
    if (read < 6)
    {
      vtprintf ("Error in vsip_cfirflt_f (0)\n"
                "unable to read row %i in %s\n", i, filename);
      fclose (fp);
      exit (1);
    }
    vsip_cvput_f (*input, i, vsip_cmplx_f (val1, val2));
    vsip_cvput_f (*output, i, vsip_cmplx_f (val3, val4));
    if (i < kernel_size)
    {
      vsip_cvput_f (*kernel, i, vsip_cmplx_f (val5, val6));
    }
  }
  fclose (fp);
}

int main(int argc, char *argv[])
{
  int ret = 0;
  int numfiles = VSIPTEST_FIRFLT_F_NUMFILES;
  char filename[120];
  int trial;
  vsip_cvview_f *tempv, *outputv;
  vsip_cvview_f *inputv, *kernelv;
  vsip_length N, D, ntimes;
  vsip_symmetry symmetry;
  int i;
  vsip_scalar_f e_real, r_real, e_imag, r_imag;
  vsip_cfir_f *fir = NULL;
  vsip_obj_state state;
  int size;

  vsip_init ((void *)0);

  error_init (argv[1]);

  for (trial=0; trial<numfiles; trial++)
  {
    sprintf (filename, "data/%s%i", VSIPTEST_CFIRFLT_F_BASENAME, 
             trial+1);
    vector_read (filename, &inputv, &outputv, &kernelv, &symmetry, &N, &D, &state, &ntimes);

    size = ceil ((float)N/(float)D);
    tempv = vsip_cvcreate_f (size, 0);
    
    if (fir == NULL)
    {
      fir = vsip_cfir_create_f (kernelv, symmetry, N, D, state, ntimes, 0);
    }

    if (!fir)
    {
      vtprintf ("Error in vsip_cfir_create_f (2)\n"
                "Could not create FIR object of N=%i D=%i "
                "ntimes=%i trial #%i\n", N, D, ntimes, trial+1);
      ret |= 1<<2;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    vsip_cfirflt_f (fir, inputv, tempv);

    for (i=0; i<size; i++)
    {
      r_real = vsip_real_f (vsip_cvget_f (tempv, i));
      r_imag = vsip_imag_f (vsip_cvget_f (tempv, i));

      e_real = vsip_real_f (vsip_cvget_f (outputv, i));
      e_imag = vsip_imag_f (vsip_cvget_f (outputv, i));

      update_error_element (sqrt((r_real-e_real)*(r_real-e_real)+(r_imag-e_imag)*(r_imag-e_imag)));

      if (fabs (r_real - e_real) > VSIPTEST_CFIRFLT_F_MAX_EPSILON ||
          fabs (r_imag - e_imag) > VSIPTEST_CFIRFLT_F_MAX_EPSILON )
      {
        vtprintf ("Error in vsip_cfirflt_f (3)\n"
                  "Element %i of result vector (%f, %f) did not match "
                  "expected value (%f, %f)\n"
                  "for N=%i D=%i ntimes=%i trial=%i\n", i, r_real, r_imag, 
				  e_real, e_imag, N, D, ntimes, trial+1);
        ret |= 1<<3;
        break;
      }
	  /* bag sign test 
      if ((r_real < 0 && e_real >= 0) || 
          (r_real == 0 && e_real != 0) ||
          (r_real > 0 && e_real <= 0) ||
          (r_imag < 0 && e_imag >= 0) || 
          (r_imag == 0 && e_imag != 0) ||
          (r_imag > 0 && e_imag <= 0) )
      {
        vtprintf ("Error in vsip_cfirflt_f (4)\n"
                  "Element %i of result vector (%f, %f) failed sign test "
                  "against expected value (%f, %f)\n"
                  "for N=%i D=%i ntimes=%i trial=%i\n", i, r_real, r_imag,
                  e_real, e_imag, N, D, ntimes, trial+1);
        ret |= 1<<4;
        break;
      }
	  */
    }
    update_error_total ();
    error_report_test ("Test #%i for N=%i D=%i ntimes=%i", trial+1, N, D, ntimes);
    if (state != VSIP_STATE_SAVE)
    {
      vsip_cfir_destroy_f (fir);
      fir = NULL;
    }
    vsip_cvalldestroy_f (tempv);
    vsip_cvalldestroy_f (kernelv);
    vsip_cvalldestroy_f (inputv);
    vsip_cvalldestroy_f (outputv);
  }
  if (fir != NULL)
  {
    vsip_cfir_destroy_f (fir);
  }
  error_finish();
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
