/* vsiptest_crfftop_f.c Version 0.000 November 4, 1999
// 
// This file contains the vsiptest_crfftop_f test functionality.
// 
// Functional Description:
//   vsip_crfftop_f does an inverse FFT form a complex view into a real view
//  
//
// Copyright November 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Revision History:
// Nov  4 1999 - Created
*/

#include"vsiptest_sigproc.h"
#include <stdio.h>

void vector_read (char *filename, vsip_cvview_f **input, 
                  vsip_vview_f **output, vsip_length *size, 
                  vsip_scalar_f *scale, vsip_length *ntimes)
{
  FILE *fp;
  int read = 0;
  int i;
  float val1, val2, val3;
  int sze, nt;
  int s2;

  fp = fopen (filename, "r");
  if (!fp)
  {
    vtprintf ("Error in vsip_crfftop_f (0)\n"
              "unable to open data file %s for reading\n", filename);
    exit(1);
  }

  read = fscanf (fp, "%i %f %i\n", &sze, scale, &nt);
  *size = sze;
  *ntimes = nt;
  s2 = sze / 2 + 1;
  if (read != 3)
  {
    vtprintf ("Error in vsip_crfftop_f (0)\n"
              "unable to read FFT parameters in %s\n", filename);
    fclose (fp);
    exit (1);
  }
  
  *output = vsip_vcreate_f (*size, 0);
  *input = vsip_cvcreate_f (s2, 0);

  if (!input || !output)
  {
    vtprintf ("Error in vsip_crfftop_f (0)\n"
              "unable to create view of size %i for %s\n", *size, filename);
    fclose (fp);
    exit(1);
  }

  for (i=0; i<*size; i++)
  {
    read = fscanf (fp, "%f,%f %f\n", &val1, &val2, &val3);
    if (read < 3)
    {
      vtprintf ("Error in vsip_crfftop_f (0)\n"
                "unable to read row %i in %s\n", i, filename);
      fclose (fp);
      exit (1);
    }
    if (i<s2)
    {
      vsip_cvput_f (*input, i, vsip_cmplx_f (val1, val2));
    }
    vsip_vput_f (*output, i, val3);
  }
  
  fclose (fp);
}

int main(int argc, char *argv[])
{
  int ret = 0;
  int numfiles = VSIPTEST_CRFFTOP_F_NUMFILES;
  char filename[120];
  int trial;
  vsip_vview_f *tempv, *outputv;
  vsip_cvview_f *inputv;
  vsip_scalar_f scale;
  vsip_length size, ntimes;
  vsip_fft_f *fft1;
  int i;
  vsip_scalar_f e_real, r_real;

  vsip_init ((void *)0);

  error_init (argv[1]);

  for (trial=0; trial<numfiles; trial++)
  {
    sprintf (filename, "data/%s%i", VSIPTEST_CRFFTOP_F_BASENAME, 
             trial+1);
    vector_read (filename, &inputv, &outputv, &size, &scale, &ntimes);
    tempv = vsip_vcreate_f (size, 0);
    fft1 = vsip_crfftop_create_f (size, scale, ntimes, 0);
    if (!fft1)
    {
      vtprintf ("Error in vsip_crfftop_create_f (1)\n"
                "Could not create FFT object of size=%i scale=%f "
                "ntimes=%i forward\n", size, scale, ntimes);
      ret |= 1<<1;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    vsip_crfftop_f (fft1, inputv, tempv);

    for (i=0; i<size; i++)
    {
      r_real = vsip_vget_f (tempv, i);
      e_real = vsip_vget_f (outputv, i);

      update_error_element (fabs(r_real-e_real));

      if (fabs (r_real - e_real) > VSIPTEST_CRFFTOP_F_MAX_EPSILON)
      {
        vtprintf ("Error in vsip_crfftop_f (3)\n"
                  "Element %i of result vector (%f) did not match "
                  "expected value (%f)\n"
                  "for size=%i scale=%f ntimes=%i forward trial=%i\n", i, r_real,
                  e_real, size, scale, ntimes, trial+1);
        ret |= 1<<3;
        break;
      }
      if ((r_real < 0 && e_real >= 0) || 
          (r_real == 0 && e_real != 0) ||
          (r_real > 0 && e_real <= 0))
      {
        vtprintf ("Error in vsip_crfftop_f (4)\n"
                  "Element %i of result vector (%f) failed sign test "
                  "against expected value (%f)\n"
                  "for size=%i scale=%f ntimes=%i trial=%i forward\n", i, r_real,
                  e_real, size, scale, ntimes, trial+1);
        ret |= 1<<4;
        break;
      }
    }
    update_error_total();
    error_report_test ("Test #%i for size=%i scale=%f ntimes=%i", trial+1, size, scale, ntimes);
    vsip_fft_destroy_f (fft1);
    vsip_valldestroy_f (tempv);
    vsip_cvalldestroy_f (inputv);
    vsip_valldestroy_f (outputv);
  }
  error_finish();
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
