/* vsiptest_rcfftop_f.c Version 0.000 November 2, 1999
// 
// This file contains the vsiptest_rcfftop_f test functionality.
// 
// Functional Description:
//   vsip_rcfftop_f does an FFT from a real view into a complex view
//  
//
// Copyright November 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Revision History:
// Nov  2 1999 - Created
*/

#include"vsiptest_sigproc.h"
#include <stdio.h>

void vector_read (char *filename, vsip_vview_f **input, 
                  vsip_cvview_f **output, vsip_length *size, 
                  vsip_scalar_f *scale, vsip_length *ntimes)
{
  FILE *fp;
  int read = 0;
  int i;
  float val1, val2, val3;
  int sze, nt;

  fp = fopen (filename, "r");
  if (!fp)
  {
    vtprintf ("Error in vsip_rcfftop_f (0)\n"
              "unable to open data file %s for reading\n", filename);
    exit(1);
  }

  read = fscanf (fp, "%i %f %i\n", &sze, scale, &nt);
  *size = sze;
  *ntimes = nt;
  if (read != 3)
  {
    vtprintf ("Error in vsip_rcfftop_f (0)\n"
              "unable to read FFT parameters in %s\n", filename);
    fclose (fp);
    exit (1);
  }
  
  *input = vsip_vcreate_f (*size, 0);
  *output = vsip_cvcreate_f (*size, 0);

  if (!input || !output)
  {
    vtprintf ("Error in vsip_rcfftop_f (0)\n"
              "unable to create view of size %i for %s\n", *size, filename);
    fclose (fp);
    exit(1);
  }

  for (i=0; i<*size; i++)
  {
    read = fscanf (fp, "%f %f,%f\n", &val1, &val2, &val3);
    if (read < 3)
    {
      vtprintf ("Error in vsip_rcfftop_f (0)\n"
                "unable to read row %i in %s\n", i, filename);
      fclose (fp);
      exit (1);
    }
    vsip_vput_f (*input, i, val1);
    vsip_cvput_f (*output, i, vsip_cmplx_f (val2, val3));
  }
  
  fclose (fp);
}

int main(int argc, char *argv[])
{
  int ret = 0;
  int numfiles = VSIPTEST_RCFFTOP_F_NUMFILES;
  char filename[120];
  int trial;
  vsip_cvview_f *ctempv, *outputv;
  vsip_vview_f *inputv;
  vsip_scalar_f scale;
  vsip_length size, ntimes;
  vsip_fft_f *fft1;
  int i;
  vsip_scalar_f e_real, e_imag, r_real, r_imag;

  vsip_init ((void *)0);

  error_init (argv[1]);

  for (trial=0; trial<numfiles; trial++)
  {
    sprintf (filename, "data/%s%i", VSIPTEST_RCFFTOP_F_BASENAME, 
             trial+1);
    vector_read (filename, &inputv, &outputv, &size, &scale, &ntimes);
    ctempv = vsip_cvcreate_f (size, 0);
    fft1 = vsip_rcfftop_create_f (size, scale, ntimes, 0);
    if (!fft1)
    {
      vtprintf ("Error in vsip_rcfftop_create_f (1)\n"
                "Could not create FFT object of size=%i scale=%f "
                "ntimes=%i forward\n", size, scale, ntimes);
      ret |= 1<<1;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    vsip_rcfftop_f (fft1, inputv, ctempv);

    for (i=0; i<size/2+1; i++)
    {
      r_real = vsip_real_f (vsip_cvget_f (ctempv, i));
      r_imag = vsip_imag_f (vsip_cvget_f (ctempv, i));

      e_real = vsip_real_f (vsip_cvget_f (outputv, i));
      e_imag = vsip_imag_f (vsip_cvget_f (outputv, i));

      update_error_element (sqrt((r_real-e_real)*(r_real-e_real)+(r_imag-e_imag)*(r_imag-e_imag)));

      if (fabs (r_real - e_real) > VSIPTEST_RCFFTOP_F_MAX_EPSILON ||
          fabs (r_imag - e_imag) > VSIPTEST_RCFFTOP_F_MAX_EPSILON)
      {
        vtprintf ("Error in vsip_rcfftop_f (3)\n"
                  "Element %i of result vector (%f, %f) did not match "
                  "expected value (%f, %f)\n"
                  "for size=%i scale=%f ntimes=%i trial=%i forward\n", i, r_real,
                  r_imag, e_real, e_imag, size, scale, ntimes, trial+1);
        ret |= 1<<3;
        break;
      }
      /* 
      if ((r_real < 0 && e_real >= 0) || 
          (r_real == 0 && e_real != 0) ||
          (r_real > 0 && e_real <= 0) ||
          (r_imag < 0 && e_imag >= 0) || 
          (r_imag == 0 && e_imag != 0) ||
          (r_imag > 0 && e_imag <= 0))
      {
        vtprintf ("Error in vsip_rcfftop_f (4)\n"
                  "Element %i of result vector (%f, %f) failed sign test "
                  "against expected value (%f, %f)\n"
                  "for size=%i scale=%f ntimes=%i trial=%i forward\n", i, r_real,
                  r_imag, e_real, e_imag, size, scale, ntimes, trial+1);
        ret |= 1<<4;
        break;
      }
      */
    }
    update_error_total();
    error_report_test ("Test #%i for size=%i scale=%f ntimes=%i", trial+1, size, scale, ntimes);
    vsip_fft_destroy_f (fft1);
    vsip_cvalldestroy_f (ctempv);
    vsip_valldestroy_f (inputv);
    vsip_cvalldestroy_f (outputv);
  }
  error_finish();
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
