/* vsiptest_vhisto_f.c Version 0.000 November 8, 1999
// 
// This file contains the vsiptest_vhisto_f test functionality.
// 
// Functional Description:
//   vsip_vhisto_f does a histogram on the input vector
//  
//
// Copyright November 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Revision History:
// Nov  8 1999 - Created
*/

#include"vsiptest_sigproc.h"
#include <stdio.h>

void vector_read (char *filename, vsip_vview_f **input, 
                  vsip_vview_f **output, 
                  vsip_length *out_length, vsip_scalar_f *min_bin,
                  vsip_scalar_f *max_bin, vsip_hist_opt *opt)
{
  FILE *fp;
  int read = 0;
  int i;
  float val1, val2;
  int in_length, ol;
  int n;
  int op;
  float maxb, minb;

  fp = fopen (filename, "r");
  if (!fp)
  {
    vtprintf ("Error in vsip_vhisto_f (0)\n"
              "unable to open data file %s for reading\n", filename);
    exit(1);
  }

  read = fscanf (fp, "%i %i %f %f %i\n", &in_length, &ol, &minb, &maxb, &op);
  *out_length = ol;
  *min_bin = minb;
  *max_bin = maxb;
  *opt = op;

  if (read != 5)
  {
    vtprintf ("Error in vsip_vhisto_f (0)\n"
              "unable to read histogram parameters in %s\n", filename);
    fclose (fp);
    exit (1);
  }
  
  *input = vsip_vcreate_f (in_length, 0);
  *output = vsip_vcreate_f (ol, 0);

  if (!input || !output)
  {
    vtprintf ("Error in vsip_vhisto_f (0)\n"
              "unable to create views of size %i,%i for %s\n", in_length, ol, filename);
    fclose (fp);
    exit(1);
  }

  if (ol > in_length)
  {
    n = ol;
  }
  else
  {
    n = in_length;
  }
  for (i=0; i<n; i++)
  {
    read = fscanf (fp, "%f %f\n", &val1, &val2);
    if (read < 2)
    {
      vtprintf ("Error in vsip_vhisto_f (0)\n"
                "unable to read row %i in %s\n", i, filename);
      fclose (fp);
      exit (1);
    }
    if (i < in_length)
    {
      vsip_vput_f (*input, i, val1);
    }
    if (i < ol)
    {
      vsip_vput_f (*output, i, val2);
    }
  }
  fclose (fp);
}

int main(int argc, char *argv[])
{
  int ret = 0;
  int numfiles = VSIPTEST_VHISTO_F_NUMFILES;
  char filename[120];
  int trial;
  vsip_vview_f *tempv=NULL, *outputv;
  vsip_vview_f *inputv;
  int i;
  vsip_scalar_f e_real, r_real;
  vsip_length out_length;
  vsip_scalar_f min_bin, max_bin;
  vsip_hist_opt opt;

  vsip_init ((void *)0);

  error_init (argv[1]);

  for (trial=0; trial<numfiles; trial++)
  {
    sprintf (filename, "data/%s%i", VSIPTEST_VHISTO_F_BASENAME, 
             trial+1);
    vector_read (filename, &inputv, &outputv, &out_length, &min_bin, &max_bin, &opt);
    
    if (opt != VSIP_HIST_ACCUM)
    {
      if (tempv) 
      {
        vsip_valldestroy_f (tempv);
      }
      tempv = vsip_vcreate_f (out_length, 0);
    }
      

    if (!tempv)
    {
      vtprintf ("Error in vsip_vhisto_f (1)\n"
                "Could not create result vector for size %i\n", out_length);
      ret |= 1<<1;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    vsip_vhisto_f (inputv, min_bin, max_bin, opt, tempv);

    for (i=0; i<out_length; i++)
    {
      r_real = vsip_vget_f (tempv, i);

      e_real = vsip_vget_f (outputv, i);

      update_error_element (fabs(e_real-r_real));

      if (fabs (r_real - e_real) > VSIPTEST_VHISTO_F_MAX_EPSILON)
      {
        vtprintf ("Error in vsip_vhisto_f (3)\n"
                  "Element %i of result vector (%f) did not match "
                  "expected value (%f)\n"
                  "for min_bin=%f max_bin=%f trial=%i\n", i, r_real, 
                   e_real, min_bin, max_bin, trial+1);
        ret |= 1<<3;
        break;
      }
      if ((r_real < 0 && e_real >= 0) || 
          (r_real == 0 && e_real != 0) ||
          (r_real > 0 && e_real <= 0) )
      {
        vtprintf ("Error in vsip_vhisto_f (4)\n"
                  "Element %i of result vector (%f) failed sign test "
                  "against expected value (%f)\n"
                  "for min_bin=%f max_bin=%f trial=%i\n", i, r_real,
                  e_real, min_bin, max_bin, trial+1);
        ret |= 1<<4;
        break;
      }
    }
    update_error_total();
    error_report_test ("Test #%i for min_bin=%f max_bin=%f opt=%i", trial+1, min_bin, max_bin, opt);
    vsip_valldestroy_f (inputv);
    vsip_valldestroy_f (outputv);
  }
  error_finish();
  vsip_valldestroy_f (tempv);
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
