/* vsiptest_vcloneview_f.c Version 0.000 May 25, 1999
// 
// This file contains the vsiptest_vcloneview_f test functionality.
// 
// Functional Description:
//   vsip_vcloneview_f clones a view
//
// Copyright May 199i, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Test that a memory allocation error causes cloneview to return NULL 
//   2) Create large complex split and interleaved blocks
//   3) for each of several view strides, offsets, and lengths:
//      a) create a view to each block
//      b) fill the views with data
//      c) clone the views
//      d) verify that the new views are not the same as the old views
//      e) delete the original views
//      f) verify that the data in the cloned views are correct
//      g) delete the cloned views
//
// Revision History:
// May 25, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_cblock_f *sblock, *iblock;
  vsip_cvview_f *cview, *cview2;
  vsip_cvview_f *sview, *sview2, *iview, *iview2;
  int            have_malloc_failer = vthas_fail_mech();
  int            ret = 0;
  vsip_cscalar_f cval;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  int            BLOCK_SIZE = VSIPTEST_CVCLONEVIEW_F_BLOCK_SIZE;
  vsip_scalar_f  *array1, *array2, *array3;

  array1 = (vsip_scalar_f *)malloc (BLOCK_SIZE*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)malloc (BLOCK_SIZE*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)malloc (BLOCK_SIZE*sizeof(vsip_scalar_f)*2);

  vsip_init ((void *)0);

  if (have_malloc_failer)
  {
    cview = vsip_cvcreate_f (1, 0);
    vtfail_next_malloc(1);
    cview2 = vsip_cvcloneview_f (cview);
    if (cview2 != NULL)
    {
        vtprintf ("Error in vsip_vcloneview_f() (1)\n"
                  "Failed clone did not return NULL\n");
        ret |= 1<<1;
    }
  }

  sblock = vsip_cblockbind_f (array1, array2, BLOCK_SIZE, 0);
  iblock = vsip_cblockbind_f (array3, NULL, BLOCK_SIZE, 0);
  
  for (i=0; i<elements(VSIPTEST_CVCLONEVIEW_F_LENGTH); i++)
  {
    offset = VSIPTEST_CVCLONEVIEW_F_OFFSET[i];
    stride = VSIPTEST_CVCLONEVIEW_F_STRIDE[i];
    length = VSIPTEST_CVCLONEVIEW_F_LENGTH[i];

    iview = vsip_cvbind_f (iblock, offset, stride, length);
    sview = vsip_cvbind_f (sblock, offset, stride, length);

    if (iview == NULL || iblock == NULL)
    {
      vtprintf ("Error in vsip_cvcloneview_f() (2)\n"
                "Failed to create %i element interleaved block and "
                "view\n", length);
      ret |= 1<<2;
    }
    if (sview == NULL || sblock == NULL)
    {
      vtprintf ("Error in vsip_cvcloneview_f() (3)\n"
                "Failed to create %i element split block and "
                "view\n", length);
      ret |= 1<<3;
    }

    vsip_cblockadmit_f (iblock, 1);
    vsip_cblockadmit_f (sblock, 1);

    for (j=0; j<length; j++)
    {
      cval.r = j * j + i;
      cval.i = j * i + j;

      vsip_cvput_f (iview, j, cval);

      cval.r = j * j * 2 + i;
      cval.i = j * i * 2 + j;

      vsip_cvput_f (sview, j, cval);
    }


    iview2 = vsip_cvcloneview_f (iview);
    sview2 = vsip_cvcloneview_f (sview);

    if (iview == iview2)
    {
      vtprintf ("Error in vsip_cvcloneview_f() (4)\n"
                "Cloned view of interleaved block is the same as the original "
                "view for "
                "o:%i s:%i l:%i\n", offset, stride, length);
      ret |= 1<<4;
    }
    if (sview == sview2)
    {
      vtprintf ("Error in vsip_cvcloneview_f() (5)\n"
                "Cloned view of split block is the same as the original "
                "view for "
                "o:%i s:%i l:%i\n", offset, stride, length);
      ret |= 1<<5;
    }

    vsip_cvdestroy_f (iview);
    vsip_cvdestroy_f (sview);

    for (j=0; j<length; j++)
    {
      cval = vsip_cvget_f (iview2, j);
      if ( vtfcomp (cval.r, j * j + i) ||
           vtfcomp (cval.i, j * i + j) )
      {
        vtprintf ("Erorr in vsip_cvcloneview_f() (6)\n"
                  "Cloned view of interleaved block "
                  "data element %i did not match original view "
                  "for o:%i s:%i l:%i(%f,%f)\n", j, offset, stride, length,
                  cval.r, cval.i);
        ret |= 1<<6;
        break;
      }
    }
    for (j=0; j<length; j++)
    {
      cval = vsip_cvget_f (sview2, j);
      if ( vtfcomp (cval.r, j * j * 2 + i) ||
           vtfcomp (cval.i, j * i * 2 + j) )
      {
        vtprintf ("Erorr in vsip_cvcloneview_f() (7)\n"
                  "Cloned view of split block "
                  "data element %i did not match original view "
                  "for o:%i s:%i l:%i(%f,%f)\n", j, offset, stride, length, 
                  cval.r, cval.i);
        ret |= 1<<7;
        break;
      }
    }
    vsip_cvdestroy_f (iview2);
    vsip_cvdestroy_f (sview2);
  }
  vsip_cblockdestroy_f (iblock);
  vsip_cblockdestroy_f (sblock);
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
