/* vsiptest_vgetblock_f.c Version 0.000 August 2, 1999
// 
// This file contains the vsiptest_vgetblock_f test functionality.
// 
// Functional Description:
//   vsip_vgetblock_f gets a pointer to the block from which a given view is
//   based 
//
// Copyright August 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) Create a user block
//      2) Create a VSIPL block
//      3) bind a view to each block
//      4) getblock on each view, and ensure that it matches the original block
//      5) create a user split complex block
//      6) create a user interleaved complex block
//      7) create a VSIPL complex block
//      8) for each block
//         a) bind a view
//         b) realview, check
//         c) imagview, check
//
// Revision History:
// August 3, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  vsip_block_f  *ublock, *vblock;
  vsip_cblock_f *uiblock, *usblock, *vcblock;
  vsip_vview_f  *vview, *uview;
  vsip_cvview_f *usview, *uiview, *vcview;
  int            ret = 0;
  vsip_scalar_f  *array, *array1, *array2, *array3;
  int            BLOCK_SIZE = VSIPTEST_VGETBLOCK_F_BLOCK_LENGTH;

  vsip_init ((void *)0);
  array = (vsip_scalar_f *)
           malloc(BLOCK_SIZE*sizeof(vsip_scalar_f));

  if (!array)
  {
    vtprintf ("Error in vsip_vgetblock_f() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  ublock = vsip_blockbind_f (array, BLOCK_SIZE, 0);
  if (!ublock)
  {
    vtprintf ("Error in vsip_vgetblock_f() (2)\n"
              "Could not bind block to user array\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_blockadmit_f (ublock, 1);

  vblock = vsip_blockcreate_f (BLOCK_SIZE, 0);
  if (!vblock)
  {
    vtprintf ("Error in vsip_vgetblock_f() (3)\n"
              "Could not create VSIPL block\n");
    ret |= 1<<3;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  vview = vsip_vbind_f (vblock, 0, 1, BLOCK_SIZE);
  if (!vview)
  {
    vtprintf ("Error in vsip_vgetblock_f() (4)\n"
              "Could not create view to all of VSIPL block\n");
    ret |= 1<<4;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  uview = vsip_vbind_f (ublock, 0, 1, BLOCK_SIZE);
  if (!uview)
  {
    vtprintf ("Error in vsip_vgetblock_f (5)\n"
              "Could not create view to all of user block\n");
    ret |= 1<<5;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*BLOCK_SIZE);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*BLOCK_SIZE);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*BLOCK_SIZE);
  
  if (!array1 || !array2 || !array3)
  {
    vtprintf ("Error in vsip_vgetblock_f() (6)\n"
              "Could not allocate data arrays for complex user blocks\n");
    ret |= 1<<6;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  vcblock = vsip_cblockcreate_f (BLOCK_SIZE, 0);
  usblock = vsip_cblockbind_f (array1, array2, BLOCK_SIZE, 0);
  uiblock = vsip_cblockbind_f (array3, NULL, BLOCK_SIZE, 0);
  
  if (!usblock)
  {
    vtprintf ("Error in vsip_vgetblock_f() (7)\n"
              "Could not bind split user complex block\n");
    ret |= 1<<7;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (!uiblock)
  {
    vtprintf ("Error in vsip_vgetblock_f() (8)\n"
              "Could not bind interleaved user complex block\n");
    ret |= 1<<8;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (!vcblock)
  {
    vtprintf ("Error in vsip_vgetblock_f() (9)\n"
              "Could not create VSIPL complex block\n");
    ret |= 1<<9;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  usview = vsip_cvbind_f (usblock, 0, 1, BLOCK_SIZE);
  uiview = vsip_cvbind_f (uiblock, 0, 1, BLOCK_SIZE);
  vcview = vsip_cvbind_f (vcblock, 0, 1, BLOCK_SIZE);

  if (!usview || !uiview || !vcview)
  {
    vtprintf ("Error in vsip_vgetblock_f() (10)\n"
              "Could not bind views to complex blocks\n");
    ret |= 1<<10;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (vsip_vgetblock_f (uview) != ublock)
  {
    vtprintf ("Error in vsip_vgetblock_f() (11)\n"
              "Value returned for user view did not match user block\n");
    ret |= 1<<11;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (vsip_vgetblock_f (vview) != vblock)
  {
    vtprintf ("Error in vsip_vgetblock_f() (12)\n"
              "Value returned for VSIPL view did not match VSIPL block\n");
    ret |= 1<<12;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (!vsip_vgetblock_f (vsip_vrealview_f (uiview)))
  {
    vtprintf ("Error in vsip_vgetblock_f() (13)\n"
              "Could not getblock for real part of user interleaved view\n");
    ret |= 1<<13;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (!vsip_vgetblock_f (vsip_vrealview_f(usview)))
  {
    vtprintf ("Error in vsip_vgetblock_f() (14)\n"
              "Could not getblock for real part of user split view\n");
    ret |= 1<<14;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (!vsip_vgetblock_f (vsip_vrealview_f (vcview)))
  {
    vtprintf ("Error in vsip_vgetblock_f() (15)\n"
              "Could not getblock for real part of VSIPL complex view\n");
    ret |= 1<<15;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (!vsip_vgetblock_f (vsip_vimagview_f (uiview)))
  {
    vtprintf ("Error in vsip_vgetblock_f() (16)\n"
              "Could not getblock for imaginary part of user interleaved view\n");
    ret |= 1<<16;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (!vsip_vgetblock_f (vsip_vimagview_f(usview)))
  {
    vtprintf ("Error in vsip_vgetblock_f() (17)\n"
              "Could not getblock for imaginary part of user split view\n");
    ret |= 1<<17;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (!vsip_vgetblock_f (vsip_vimagview_f (vcview)))
  {
    vtprintf ("Error in vsip_vgetblock_f() (18)\n"
              "Could not getblock for imaginary part of VSIPL complex view\n");
    ret |= 1<<18;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  vsip_vdestroy_f (vview);
  vsip_vdestroy_f (uview);
  vsip_cvdestroy_f (usview);
  vsip_cvdestroy_f (uiview);
  vsip_cvdestroy_f (vcview);
  vsip_cblockdestroy_f (vcblock);
  vsip_cblockdestroy_f (usblock);
  vsip_cblockdestroy_f (uiblock);
  vsip_blockdestroy_f (vblock);
  vsip_blockdestroy_f (ublock);
  free (array);
  free (array1);
  free (array2);
  free (array3);
  
  report (ret); 
    vsip_finalize ((void *)0);
  return (ret != 0);
}
